// ****************** Copyright (c) 2013 by Gigaset Communications GmbH ******
// Filename : tdav_codec_h263_hw_vpu.cxx
// Owner    : Mohammed Hasan [STC]
//-------------------------------------------------------------------------
// Defintion : H.263 codec plugin using FSL OMX HW codec for encoding and decoding.
// RTP payloader follows RFC 4629 for H263+ and RFC 2190 for H263.
//-------------------------------------------------------------------------
// 26-07-2013, Gigaset Communications GmbH
// Added FSL OMX HW H.263 codec plugin to Doubango Framework
//*************************************************************************

/*
* This file is part of Open Source Doubango Framework.
*
* DOUBANGO is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* DOUBANGO is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with DOUBANGO.
*
*/

#include "tinydav/codecs/h263/tdav_codec_h263_hw_vpu.h"

#if HAVE_FFMPEG

#include "tinydav/video/tdav_converter_video.h"

#include "tinyrtp/rtp/trtp_rtp_packet.h"

#include "tnet_endianness.h"

#include "tinymedia/tmedia_params.h"


#include "tsk_string.h"
#include "tsk_time.h"
#include "tsk_memory.h"
#include "tsk_debug.h"
#include "tsk_timer.h"

#ifndef INT64_C
#       define INT64_C(c) (c ## LL)
#       define UINT64_C(c) (c ## ULL)
#endif
#ifdef __cplusplus
extern "C" {
#endif
#if HAVE_FFMPEG
#   include <libavcodec/avcodec.h>
#   if (LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(51, 35, 0))
#       include <libavutil/opt.h>
#   endif
#endif
#ifdef __cplusplus
}
#endif

#define TDAV_H263_GOP_SIZE_IN_SECONDS		25
#define RTP_PAYLOAD_SIZE	750
#define RTP_PAYLOAD_CORRECTION 5

#define H263P_HEADER_SIZE		2
#define H263_HEADER_MODE_A_SIZE 4
#define H263_HEADER_MODE_B_SIZE 8
#define H263_HEADER_MODE_C_SIZE 12

#define tdav_codec_h263p_hw_vpu_set tdav_codec_h263_hw_vpu_set
#define tdav_codec_h263p_hw_vpu_open tdav_codec_h263_hw_vpu_open
#define tdav_codec_h263p_hw_vpu_close tdav_codec_h263_hw_vpu_close
#define tdav_codec_h263p_hw_vpu_encode tdav_codec_h263_hw_vpu_encode
#define tdav_codec_h263p_hw_vpu_decode tdav_codec_h263_hw_vpu_decode
#define tdav_codec_h263p_hw_vpu_sdp_att_match tdav_codec_h263_hw_vpu_sdp_att_match
#define tdav_codec_h263p_hw_vpu_sdp_att_get tdav_codec_h263_hw_vpu_sdp_att_get

#define tdav_codec_h263pp_hw_vpu_set tdav_codec_h263_hw_vpu_set
#define tdav_codec_h263pp_hw_vpu_open tdav_codec_h263_hw_vpu_open
#define tdav_codec_h263pp_hw_vpu_close tdav_codec_h263_hw_vpu_close
#define tdav_codec_h263pp_hw_vpu_encode tdav_codec_h263_hw_vpu_encode
#define tdav_codec_h263pp_hw_vpu_decode tdav_codec_h263_hw_vpu_decode
#define tdav_codec_h263pp_hw_vpu_sdp_att_match tdav_codec_h263_hw_vpu_sdp_att_match
#define tdav_codec_h263pp_hw_vpu_sdp_att_get tdav_codec_h263_hw_vpu_sdp_att_get

#define TDAV_CODEC_H263(self) ((tdav_codec_h263_hw_vpu_t*)(self))

typedef enum tdav_codec_h263_type_e
{
	tdav_codec_h263_1996,
	tdav_codec_h263_1998,
	tdav_codec_h263_2000,
}
tdav_codec_h263_type_t;

//Android includes
#include <android/log.h>
#include <cutils/properties.h>
#include "OMX_Implement.h"
#include "fsl_osal.h"
#include "Mem.h"
#include "Log.h"

#include "OMX_Core.h"
#include "OMX_Component.h"
#include "Queue.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <pthread.h>

//#define OMX_H263_DEBUG
#define  LOG_TAG    "OMX H263"

#ifdef OMX_H263_DEBUG
#define  TSK_DEBUG_INFO(...)  __android_log_print(ANDROID_LOG_INFO,LOG_TAG,__VA_ARGS__)
#define  TSK_DEBUG_ERROR(...)  __android_log_print(ANDROID_LOG_ERROR,LOG_TAG,__VA_ARGS__)
#define  TSK_DEBUG_WARN(...)  __android_log_print(ANDROID_LOG_WARN,LOG_TAG,__VA_ARGS__)
#else
#define  TSK_DEBUG_INFO(...)
#define  TSK_DEBUG_ERROR(...)  __android_log_print(ANDROID_LOG_ERROR,LOG_TAG,__VA_ARGS__)
#define  TSK_DEBUG_WARN(...)  __android_log_print(ANDROID_LOG_WARN,LOG_TAG,__VA_ARGS__)
#endif // OMX_H263_DEBUG

#define MAX_PORT_NUM 8
#define MAX_PORT_BUFFER 32
#define MAX_QUEUE_SIZE 30
#define MUL_FACTOR 1000
#define SLEEP_TIME 10000
#define TIME_BASE_NUM 1
#define BIT_RATE_CAL_PARAM1 256
#define BIT_RATE_CAL_PARAM2 320
#define BIT_RATE_CAL_PARAM3 240
#define ENC_MIN_PORT_BUF 1
#define ENC_MAX_PORT_BUF 3

#define VPU_ENC_COMP_NAME_AVCENC    "OMX.Freescale.std.video_encoder.h263.hw-based"
#define VPU_ENC_COMP_ROLE_AVCENC    "video_encoder.h263"
#define VPU_DEC_COMP_NAME_AVCDEC    "OMX.Freescale.std.video_decoder.h263.hw-based"
#define VPU_DEC_COMP_ROLE_AVCDEC    "video_decoder.h263"

#define Align(ptr,align)        (((OMX_U32)(ptr)+(align)-1)/(align)*(align))
#define FRAME_ALIGN             (32)
#define MAX_ENC_BUF_TO_KEEP 4
#define MAX_DEC_BUF_TO_KEEP 2
#define LOW_MOTION  1
#define MED_MOTION  2
#define HIGH_MOTION 3

#define DUMMY_720P_FRAME_PATH "/system/media/dummyframe-720p.h263"
#define DUMMY_VGA_FRAME_PATH "/system/media/dummyframe-vga.h263"
#define DUMMY_QVGA_FRAME_PATH "/system/media/dummyframe-qvga.h263"
#define HT_720P 720
#define HT_VGA 480
#define HT_QVGA 240
#define TH263_WAIT_FOR_PAUSE 1000 /* Time (ms) to wait before entering in PAUSE state*/
#define TH263_WAIT_FOR_RESET 20 /* Time in ms to wait before resetting decoder */

#define MAX_IN_ENC_RB_CAP 4 /* Maximum number of buffers to hold in enc i/p ring buffer (raw buffer)*/
#define MAX_IN_DEC_RB_CAP 8 /* Maximum number of buffers to hold in ring buffer (encoded buffer) */
#define MAX_OUT_DEC_RB_CAP 4 /* Maximum number of buffers to hold in ring buffer (decoded buffer)*/

/* I-Frame request frequency in seconds */
#define HIGH_GOP_SIZE_IN_SEC 0 /* Use it for low resolution and gets i-frames @ 0 sec */
#define MED_GOP_SIZE_IN_SEC 4 /* Use it for medium resolution and gets i-frames @ 4 secs */
#define LOW_GOP_SIZE_IN_SEC 14 /* Use it for high resolution and gets i-frames @ 14 secs*/
#define DEF_GOP_SIZE_IN_SEC 14 /* Use it for high resolution and gets i-frames @ 14 secs*/

#define MAX_GOP_SIZE_ALLOWED 32766
#define MAX_BR_FACT 10

#define VIDEO_DUMP

#ifdef VIDEO_DUMP
FILE *fpEncodedBufferh263 = NULL;
FILE *fpDecodedBufferh263 = NULL;
#endif

typedef enum {
    EVENT = 0,
    EMPTY_DONE,
    FILL_DONE,
    EXIT
}MSG_TYPE;

enum {
    kPortIndexInput  = 0,
    kPortIndexOutput = 1
};

typedef struct {
    OMX_EVENTTYPE eEvent;
    OMX_U32 nData1;
    OMX_U32 nData2;
    OMX_PTR pEventData;
}EVENT_TYPE;

typedef struct {
    OMX_BUFFERHEADERTYPE* pBuffer;
}BUFFER;

typedef struct _MSG {
    OMX_HANDLETYPE hComponent;
    MSG_TYPE type;
    union {
        EVENT_TYPE event;
        BUFFER buffer;
    }data;
}MSG;

typedef struct _var_buffer {
    void* buffer;
    int size;
} var_buffer;

typedef struct _ring_buffer {
    var_buffer * buffer;
    void * buffer_end;
    int capacity;
    int size;
    int count;
    void *head;
    void *tail;
    pthread_mutex_t rw_mutex;
} ring_buffer;

/** H.263-1996 codec */
typedef struct tdav_codec_h263_hw_vpu_s
{
	TMEDIA_DECLARE_CODEC_VIDEO;

	tdav_codec_h263_type_t type;

	struct{
		uint8_t* ptr;
		tsk_size_t size;
	} rtp;

        tsk_bool_t timer_started;
        tsk_timer_manager_handle_t *h_timer;

        // Timer
        struct {
            tsk_timer_id_t id;
            int64_t timeout;
        } timer_idle;

        struct {
            tsk_timer_id_t id;
            int64_t timeout;
        } timer_reset_dec;

        bool video_dump_to_file;
        bool disable_dummy_buffer;
        int idle_timeout_period;
        int reset_timeout_period;

        // Encoder
        struct{
                AVCodecContext* context;
                void* buffer;
                void* dummy_buffer;
                tsk_size_t dummy_buffer_size;
                void* dummy_decode_buffer;
                tsk_size_t dummy_decode_buffer_size;
                OMX_BOOL decoded_first_frame;
                OMX_BOOL encoded_first_frame;
                tsk_size_t buffer_size;
                tsk_bool_t force_idr;
                int32_t quality; // [1-31]

                //hw_vpu
                OMX_COMPONENTTYPE *hComponent;
                OMX_STRING name;
                OMX_STRING role;
                OMX_U32 nPorts;
                OMX_BOOL bAllocater[MAX_PORT_NUM];
                OMX_U32 nBufferHdr[MAX_PORT_NUM];
                OMX_DIRTYPE PortDir[MAX_PORT_NUM];
                OMX_BUFFERHEADERTYPE *pBufferHdr[MAX_PORT_NUM][MAX_PORT_BUFFER];
                OMX_BUFFERHEADERTYPE *pFrameBuffer;
                Queue *pMsgQ;
                EVENT_TYPE sCmdDone;
                fsl_osal_ptr pThreadId;
                OMX_BOOL bInEos;
                OMX_BOOL bError;
                OMX_BOOL bHoldBuffers;
                OMX_BOOL bInitailBufferSubmit;
                OMX_BOOL bThreadRunning;
                OMX_BOOL bClosedEncoder;
                OMX_BOOL source_done;
                OMX_S32 width;
                OMX_S32 height;
                OMX_S32 bitrate;
                OMX_S32 inBufEmptyCount;
                OMX_S32 encPacketLoss;
                uint8_t *prev_frame;
                OMX_U32 maxBufInQueue;
                OMX_BUFFERHEADERTYPE *dummy_frame;
                OMX_S32 bitrate_fact;
                tsk_bool_t enc_started;
                //OMX_VIDEO_AVCPROFILETYPE mProfile;
                //OMX_VIDEO_AVCLEVELTYPE mLevel;
                int in_ring_buf_cap;
                ring_buffer *in_ring_buf; // Raw data from camera source
                uint32_t gop_size;
        } encoder;

        // decoder
        struct{
                AVCodecContext* context;
                void* accumulator;
                void* crop_buffer;
                tsk_size_t decode_frame_size;
                uint8_t ebit;
                tsk_size_t accumulator_pos;
                uint16_t last_seq;

                //hw_vpu
                OMX_COMPONENTTYPE *hComponent;
                OMX_STRING name;
                OMX_STRING role;
                OMX_U32 nPorts;
                OMX_BOOL bAllocater[MAX_PORT_NUM];
                OMX_U32 nBufferHdr[MAX_PORT_NUM];
                OMX_DIRTYPE PortDir[MAX_PORT_NUM];
                OMX_BUFFERHEADERTYPE *pBufferHdr[MAX_PORT_NUM][MAX_PORT_BUFFER];
                OMX_BUFFERHEADERTYPE *pFrameBuffer;
                Queue *pMsgQ;
                EVENT_TYPE sCmdDone;
                fsl_osal_ptr pThreadId;
                OMX_BOOL bInEos;
                OMX_BOOL bError;
                OMX_BOOL bHoldBuffers;
                OMX_BOOL bInitailBufferSubmit;
                OMX_BOOL bThreadRunning;
                OMX_BOOL bClosedDecoder;
                OMX_BOOL source_done;
                OMX_S32 decPacketLoss;
                uint8_t *prev_frame;
                OMX_S32 cropLeft;
                OMX_S32 cropTop;
                OMX_U32 cropRight;
                OMX_U32 cropBottom;
                OMX_U32 maxBufInQueue;
                OMX_BOOL chromaInterleave;
                tsk_bool_t dec_started;
                tsk_bool_t decoder_reset;
                int in_ring_buf_cap;
                int out_ring_buf_cap;
                ring_buffer *in_ring_buf; // Encoded data from NW
                ring_buffer *out_ring_buf; // Decoded (raw) data for display
                OMX_S32 inBufEmptyCount;
        } decoder;
}
tdav_codec_h263_hw_vpu_t;

#define TDAV_DECLARE_CODEC_H263 tdav_codec_h263_hw_vpu_t __codec_h263__

static int tdav_codec_h263_init(tdav_codec_h263_hw_vpu_t* self, tdav_codec_h263_type_t type, enum CodecID encoder, enum CodecID decoder);
static int tdav_codec_h263_deinit(tdav_codec_h263_hw_vpu_t* self);
static int tdav_codec_h263_open_encoder(tdav_codec_h263_hw_vpu_t* self);
static int tdav_codec_h263_open_decoder(tdav_codec_h263_hw_vpu_t* self);
static int tdav_codec_h263_close_encoder(tdav_codec_h263_hw_vpu_t* self);
static int tdav_codec_h263_close_decoder(tdav_codec_h263_hw_vpu_t* self);

static int hw_vpu_open_encoder(tdav_codec_h263_hw_vpu_t* self);
static int hw_vpu_open_decoder(tdav_codec_h263_hw_vpu_t* self);
static int _tdav_session_h263_timer_cb(const void* arg, tsk_timer_id_t timer_id);
static int reset_decoder(tdav_codec_h263_hw_vpu_t *h263);
static void start_reset_dec_timer(tdav_codec_h263_hw_vpu_t *h263);
static OMX_ERRORTYPE open_fsl_omx_decoder(tdav_codec_h263_hw_vpu_t* h263);
static int reset_decoder_config(tdav_codec_h263_hw_vpu_t *h263, int width, int height);
static bool set_bitrate_fac_GOP_size(tdav_codec_h263_hw_vpu_t *self);

/** H.263-1998 codec */
typedef struct tdav_codec_h263p_s
{
	TDAV_DECLARE_CODEC_H263;
}
tdav_codec_h263p_hw_vpu_t;

/** H.263-2000 codec */
typedef struct tdav_codec_h263pp_s
{
	TDAV_DECLARE_CODEC_H263;
}
tdav_codec_h263pp_hw_vpu_t;

//Decoder OMX call backs
OMX_ERRORTYPE gm_H263DecEventHandler(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_EVENTTYPE eEvent,
        OMX_IN OMX_U32 nData1,
        OMX_IN OMX_U32 nData2,
        OMX_IN OMX_PTR pEventData);

OMX_ERRORTYPE gm_H263DecEmptyBufferDone(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_BUFFERHEADERTYPE* pBuffer);

OMX_ERRORTYPE gm_H263DecFillBufferDone(
        OMX_OUT OMX_HANDLETYPE hComponent,
        OMX_OUT OMX_PTR pAppData,
        OMX_OUT OMX_BUFFERHEADERTYPE* pBuffer);

OMX_CALLBACKTYPE gH263DecCallBacks =
{
    gm_H263DecEventHandler,
    gm_H263DecEmptyBufferDone,
    gm_H263DecFillBufferDone
};

//Decoder OMX functions
OMX_ERRORTYPE read_frame(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr);
OMX_ERRORTYPE write_data(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr);
OMX_ERRORTYPE WaitCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData);
OMX_ERRORTYPE SendCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData,
        OMX_BOOL bSync);
OMX_U32 get_component_ports_h263(OMX_HANDLETYPE hComponent);
OMX_ERRORTYPE unload_component(tdav_codec_h263_hw_vpu_t *h263);
OMX_ERRORTYPE prepare_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE free_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE Load2Idle(tdav_codec_h263_hw_vpu_t *h263);
OMX_ERRORTYPE Idle2Load(tdav_codec_h263_hw_vpu_t *h263);
OMX_ERRORTYPE StateTrans(tdav_codec_h263_hw_vpu_t *h263, OMX_STATETYPE eState);
OMX_ERRORTYPE port_disable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE port_enable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE process_event(tdav_codec_h263_hw_vpu_t *self, MSG *pMsg);
static void * omx_process_thread(void *ptr);
OMX_ERRORTYPE start_data_process(tdav_codec_h263_hw_vpu_t *self);
OMX_ERRORTYPE wait_eos(tdav_codec_h263_hw_vpu_t *self);

static int close_hw_vpu_decoder(tdav_codec_h263_hw_vpu_t *self);
static int hw_vpu_decode(tdav_codec_h263_hw_vpu_t* self, const AVPacket *avpkt);
static void ProcessBufferForCrop(tdav_codec_h263_hw_vpu_t* self, uint8_t* srcPtr, uint8_t* desPtr);
static void ProcessBuffer(tdav_codec_h263_hw_vpu_t* self, uint8_t* srcPtr, uint8_t* desPtr, int size);
static void saveNV12ImageHelper(tdav_codec_h263_hw_vpu_t* self, uint8_t *pYuv, uint8_t *buf);
static void copyDecodedData(tdav_codec_h263_hw_vpu_t* self, void** out_data, int size);

// Encoder OMX call backs
OMX_ERRORTYPE gm_H263EncEventHandler(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_EVENTTYPE eEvent,
        OMX_IN OMX_U32 nData1,
        OMX_IN OMX_U32 nData2,
        OMX_IN OMX_PTR pEventData);

OMX_ERRORTYPE gm_H263EncEmptyBufferDone(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_BUFFERHEADERTYPE* pBuffer);

OMX_ERRORTYPE gm_H263EncFillBufferDone(
        OMX_OUT OMX_HANDLETYPE hComponent,
        OMX_OUT OMX_PTR pAppData,
        OMX_OUT OMX_BUFFERHEADERTYPE* pBuffer);

OMX_CALLBACKTYPE gH263EncCallBacks =
{
    gm_H263EncEventHandler,
    gm_H263EncEmptyBufferDone,
    gm_H263EncFillBufferDone
};

//Encoder OMX functions
OMX_ERRORTYPE read_enc_frame(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr);
OMX_ERRORTYPE write_enc_data(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr);
OMX_ERRORTYPE EncWaitCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData);
OMX_ERRORTYPE EncSendCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData,
        OMX_BOOL bSync);
OMX_U32 get_enc_component_ports_h263(OMX_HANDLETYPE hComponent);
OMX_ERRORTYPE unload_enc_component(tdav_codec_h263_hw_vpu_t *h263);
OMX_ERRORTYPE prepare_enc_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE free_enc_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE EncLoad2Idle(tdav_codec_h263_hw_vpu_t *h263);
OMX_ERRORTYPE EncIdle2Load(tdav_codec_h263_hw_vpu_t *h263);
OMX_ERRORTYPE EncStateTrans(tdav_codec_h263_hw_vpu_t *h263, OMX_STATETYPE eState);
OMX_ERRORTYPE enc_port_disable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE enc_port_enable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex);
OMX_ERRORTYPE enc_process_event(tdav_codec_h263_hw_vpu_t *self, MSG *pMsg);
static void * enc_omx_process_thread(void *ptr);
OMX_ERRORTYPE enc_start_data_process(tdav_codec_h263_hw_vpu_t *self);
OMX_ERRORTYPE enc_wait_eos(tdav_codec_h263_hw_vpu_t *self);
OMX_ERRORTYPE request_intraVOP(tdav_codec_h263_hw_vpu_t *self);
static int close_hw_vpu_encoder(tdav_codec_h263_hw_vpu_t *self);
static int hw_vpu_encode(tdav_codec_h263_hw_vpu_t* self, const AVPacket *avpkt);
static void free(tdav_codec_h263_hw_vpu_t *h263);

static void tdav_codec_h263_rtp_callback(tdav_codec_h263_hw_vpu_t *self, const void *data, tsk_size_t size, tsk_bool_t marker);
static void tdav_codec_h263p_rtp_callback(tdav_codec_h263_hw_vpu_t *self, const void *data, tsk_size_t size, tsk_bool_t frag, tsk_bool_t marker);

static void tdav_codec_h263_encap(const tdav_codec_h263_hw_vpu_t* h263, const uint8_t* pdata, tsk_size_t size);

static void ring_buffer_free(ring_buffer *var_rb)
{
    TSK_DEBUG_INFO("free");
    if (var_rb == NULL) {
        TSK_DEBUG_ERROR("Invalid parameter");
        return;
    }

    if (var_rb) {
        if (var_rb->buffer) {
            if (var_rb->buffer->buffer) {
                FSL_FREE(var_rb->buffer->buffer);
                var_rb->buffer->buffer = NULL;
                var_rb->buffer->size = 0;
            }
            FSL_FREE(var_rb->buffer);
            var_rb->buffer = NULL;
        }
        FSL_FREE(var_rb);
        var_rb = NULL;
    }
}

static ring_buffer* ring_buffer_init(int cap, int size)
{
    TSK_DEBUG_INFO("ring_buffer_init with capacity %d, size %d", cap, size);
    ring_buffer *var_rb = NULL;

    var_rb = (ring_buffer*) FSL_MALLOC(sizeof(ring_buffer));
    if (var_rb == NULL) {
        TSK_DEBUG_ERROR("Error allocating memory for var_rb");
        return var_rb;
    }

    var_rb->buffer = (var_buffer*) FSL_MALLOC(sizeof(var_buffer));
    if (var_rb->buffer == NULL) {
        TSK_DEBUG_ERROR("Error allocating memory for var_rb buffer");
        FSL_FREE(var_rb);
        var_rb = NULL;
        return var_rb;
    }

    var_rb->buffer->buffer = (void*) FSL_MALLOC(cap * size);
    if (var_rb->buffer->buffer == NULL) {
        TSK_DEBUG_ERROR("Error allocating memory for var_rb buffer->buffer");
        FSL_FREE(var_rb->buffer);
        var_rb->buffer = NULL;
        FSL_FREE(var_rb);
        var_rb = NULL;
        return var_rb;
    }

    TSK_DEBUG_INFO("ring_buffer_init success with size %d", cap*size);
    var_rb->buffer_end = (char*) (var_rb->buffer->buffer) + (cap * size);
    var_rb->capacity = cap;
    var_rb->size = size;
    var_rb->count = 0;
    var_rb->head = var_rb->tail = var_rb->buffer->buffer;
    var_rb->rw_mutex = PTHREAD_MUTEX_INITIALIZER;
    return var_rb;
}

static int ring_buffer_push(ring_buffer *var_rb, const void *data, int size)
{
    TSK_DEBUG_INFO("var push size %d", size);
    if (!var_rb || !data) {
        TSK_DEBUG_ERROR("var_push:Invalid param");
        return -1;
    }

    if (var_rb->count >= var_rb->capacity) {
        TSK_DEBUG_WARN("Overwriting var ring buffer count %d, cap %d",
                        var_rb->count, var_rb->capacity);
        var_rb->count = var_rb->capacity;
    }

    if (size > var_rb->size) {
        TSK_DEBUG_ERROR("var_push: invalid size: req %d, allocd %d", size, var_rb->size);
        return -1;
    }

    memcpy(var_rb->head, data, size);
    // Lock mutex
    pthread_mutex_lock(&var_rb->rw_mutex);
    var_rb->buffer->size = size;
    var_rb->head = (char*) var_rb->head + var_rb->size;//Increment tail by fixed size (Max buff size)
    if (var_rb->head == var_rb->buffer_end)
        var_rb->head = var_rb->buffer->buffer;

    var_rb->count++;
    // Unlock mutex
    pthread_mutex_unlock(&var_rb->rw_mutex);
    return 0;
}

static void ring_buffer_pop(ring_buffer *var_rb, void *data, int *size, bool realloc)
{
    if (!var_rb || !data) {
        TSK_DEBUG_ERROR("var pop:Invalid param");
        *size = 0;
        return;
    }

    // Lock mutex
    pthread_mutex_lock(&var_rb->rw_mutex);
    if (var_rb->count == 0) {
        TSK_DEBUG_WARN("No data in ring buffer");
        *size = 0;
        // Unlock mutex
        pthread_mutex_unlock(&var_rb->rw_mutex);
        return;
    }
    // Unlock mutex
    pthread_mutex_unlock(&var_rb->rw_mutex);
    TSK_DEBUG_INFO("Pop: Size available %d ", var_rb->buffer->size);

    // Re-allocate data, if requested size < available size
    if ((*size > 0) && (*size < var_rb->buffer->size)) {
        if (realloc) {
            if (!(data = tsk_realloc(data, var_rb->buffer->size))) {
                TSK_DEBUG_ERROR("ring_buffer_pop: Failed to re-allocate buffer");
                return;
            }
        } else {
            TSK_DEBUG_ERROR("Buffer overflow");
            *size = 0;
            return;
        }
    }

    memcpy(data, var_rb->tail, var_rb->buffer->size);
    *size = var_rb->buffer->size;

    // Lock mutex
    pthread_mutex_lock(&var_rb->rw_mutex);
    var_rb->tail = (char*) var_rb->tail + var_rb->size;//Increment tail by fixed size (Max buff size)
    if (var_rb->tail == var_rb->buffer_end)
        var_rb->tail = var_rb->buffer->buffer;

    var_rb->count--;
    // Unlock mutex
    pthread_mutex_unlock(&var_rb->rw_mutex);
}

/* ============ Common To all H263 codecs ================= */

static int tdav_codec_h263_hw_vpu_set(tmedia_codec_t* self, const tmedia_param_t* param)
{
	tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;
	if(!self->opened){
		TSK_DEBUG_ERROR("Codec not opened");
		return -1;
	}
	if(param->value_type == tmedia_pvt_int32){
		if(tsk_striequals(param->key, "action")){
			tmedia_codec_action_t action = (tmedia_codec_action_t)TSK_TO_INT32((uint8_t*)param->value);
			switch(action){
				case tmedia_codec_action_encode_idr:
					{
						h263->encoder.force_idr = tsk_true;
						break;
					}
				case tmedia_codec_action_bw_down:
					{
						h263->encoder.quality = TSK_CLAMP(1, (h263->encoder.quality + 1), 31);
						h263->encoder.context->global_quality = FF_QP2LAMBDA * h263->encoder.quality;
						break;
					}
				case tmedia_codec_action_bw_up:
					{
						h263->encoder.quality = TSK_CLAMP(1, (h263->encoder.quality - 1), 31);
						h263->encoder.context->global_quality = FF_QP2LAMBDA * h263->encoder.quality;
						break;
					}
			}
			return 0;
		}
	}
	return -1;
}

int tdav_codec_h263_init(tdav_codec_h263_hw_vpu_t* self, tdav_codec_h263_type_t type, enum CodecID encoder, enum CodecID decoder)
{
    int ret = 0;

    if(!self){
        TSK_DEBUG_ERROR("Invalid parameter");
        return -1;
    }

    /* allocations MUST be done by open() */
    return ret;
}

int tdav_codec_h263_deinit(tdav_codec_h263_hw_vpu_t* self)
{
    if(!self){
        TSK_DEBUG_ERROR("Invalid parameter");
        return -1;
    }

    // FFMpeg resources are destroyed by close()
    TSK_FREE(self->rtp.ptr);
    self->rtp.size = 0;

    return 0;
}

/* ============ H.263-1996 Plugin interface ================= */

//
//	H.263-1996 object definition
//
static int tdav_codec_h263_hw_vpu_open(tmedia_codec_t* self)
{
    int ret;
    tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;

    if(!h263){
        TSK_DEBUG_ERROR("Invalid parameter");
        return -1;
    }

    // read property
    char propValue[PROPERTY_VALUE_MAX];
    if (property_get("video_dump_enable", propValue, NULL)) {
        h263->video_dump_to_file = (bool) atoi(propValue);
    } else {
        h263->video_dump_to_file = false;
    }

    // By default, it is set to disable
    if (property_get("disable_dummy_buffer", propValue, NULL)) {
        h263->disable_dummy_buffer = (bool) atoi(propValue);
    } else {
        h263->disable_dummy_buffer = true;
    }

    if (property_get("idle_time_out_period", propValue, NULL)) {
        h263->idle_timeout_period = atoi(propValue);
        if (h263->idle_timeout_period <= 0) {
            h263->idle_timeout_period = TH263_WAIT_FOR_PAUSE;
        }
    }

    if (property_get("reset_time_out_period", propValue, NULL)) {
        h263->reset_timeout_period = atoi(propValue);
        if (h263->reset_timeout_period <= 0) {
            h263->reset_timeout_period = TH263_WAIT_FOR_RESET;
        }
    }

    TSK_DEBUG_INFO("dump %d, disable dummy buffer %d, idle and reset timeout %d & %d",
                    h263->video_dump_to_file, h263->disable_dummy_buffer,
                    h263->idle_timeout_period, h263->reset_timeout_period);

    // hw_vpu
    int err = OMX_Init();
    if (err < 0) {
        TSK_DEBUG_ERROR("Failed to initialize OMX_Init err %d", err);
    }

    /* the caller (base class) already checked that the codec is not opened */

    //	Encoder
    if((ret = tdav_codec_h263_open_encoder(h263))){
        return ret;
    }

    //	Decoder
    if((ret = tdav_codec_h263_open_decoder(h263))){
        return ret;
    }

#ifdef VIDEO_DUMP
    if (h263->video_dump_to_file) {
        fpEncodedBufferh263 = fopen("/data/cmbs/encoded-stream.h263", "w");
        if (fpEncodedBufferh263 == NULL) {
           TSK_DEBUG_ERROR("Failed to open file");
        }
        fpDecodedBufferh263 = fopen("/data/cmbs/decoded-stream.h263", "w");
        if (fpDecodedBufferh263 == NULL) {
           TSK_DEBUG_ERROR("Failed to open file");
        }
    }
#endif

    h263->encoder.enc_started = tsk_false;
    h263->decoder.dec_started = tsk_false;

    // Create timer for pasuing codec after timeout period
    if (!(h263->h_timer = tsk_timer_manager_create())) {
        TSK_DEBUG_ERROR("Failed to create timer manager for h263");
        //ignore this error
        return 0;
    }

    if ((ret = tsk_timer_manager_start(h263->h_timer)) != 0) {
        TSK_DEBUG_ERROR("Failed to start timer");
        //Can ignore this error
        return 0;
    }

    h263->timer_started = (ret == 0);

    if (h263->timer_started) {
        h263->timer_idle.timeout = h263->idle_timeout_period;
        h263->timer_idle.id = tsk_timer_manager_schedule(h263->h_timer, h263->timer_idle.timeout, _tdav_session_h263_timer_cb, h263);
        TSK_DEBUG_INFO("Timer sheduled with timeout value %d ms", h263->idle_timeout_period);
    }

    return ret;
}

static int tdav_codec_h263_hw_vpu_close(tmedia_codec_t* self)
{
    TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_close IN \n");
    tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;
    int ret;

    if(!h263){
        TSK_DEBUG_ERROR("Invalid parameter");
        return -1;
    }

    /* the caller (base class) already checked that the codec is opened */

    //	Encoder
    ret = tdav_codec_h263_close_encoder(h263);

    //	Decoder
    ret = tdav_codec_h263_close_decoder(h263);

    ret = OMX_Deinit();
    if (ret < 0) {
        TSK_DEBUG_ERROR("Failed to de-initialize OMX_Deinit err %d\n", ret);
    }

    if (h263->h_timer != tsk_null) {
        tsk_timer_manager_stop(h263->h_timer);
        h263->h_timer = tsk_null;
    }

#ifdef VIDEO_DUMP
    if (h263->video_dump_to_file) {
        if (fpEncodedBufferh263) {
            fclose(fpEncodedBufferh263);
            fpEncodedBufferh263 = NULL;
        }
        if (fpDecodedBufferh263) {
            fclose(fpDecodedBufferh263);
            fpDecodedBufferh263 = NULL;
        }
    }
#endif

    TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_close OUT \n");
    return ret;
}

static tsk_size_t tdav_codec_h263_hw_vpu_encode(tmedia_codec_t* self, const void* in_data, tsk_size_t in_size, void** out_data, tsk_size_t* out_max_size)
{
    int ret = 0;

    //TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_encode: in_size %d, out_max_size %d\n", in_size, *out_max_size);
    int size;

    tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;

    if(!self || !in_data || !in_size || !out_data){
        TSK_DEBUG_ERROR("Invalid parameter");
        return 0;
    }

    // Check if encoder is in Pause state
    if (!h263->encoder.enc_started) {
        // Change state to Executing, if it was in Pause state
        EncStateTrans(h263, OMX_StateExecuting);
        h263->encoder.enc_started = tsk_true;
    }

    if (h263->encoder.force_idr) {
        TSK_DEBUG_INFO("Requesting I-Frame");
        request_intraVOP(h263);
    }

    // Encode data
    AVPacket packet;

    /* decode the picture */
    av_init_packet(&packet);
    packet.size = in_size;
    packet.data = (uint8_t*)in_data;

    if (packet.data) {
        //ret = hw_vpu_encode(h263, h263->encoder.buffer, h263->encoder.buffer_size, &packet);
        ret = hw_vpu_encode(h263, &packet);

        // tdav_codec_h263_encap will be called from write_enc_data(),
        // when encoding finished.
        if(ret > 0){
            //tdav_codec_h263_encap(h263, (const uint8_t*)h263->encoder.buffer, (tsk_size_t)ret);
            TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_encode: invalid case");
        }
    }
    h263->encoder.force_idr = tsk_false;

    //TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_encode OUT\n");
    return 0;
}

static tsk_size_t tdav_codec_h263_hw_vpu_decode(tmedia_codec_t* self, const void* in_data, tsk_size_t in_size, void** out_data, tsk_size_t* out_max_size, const tsk_object_t* proto_hdr)
{
    //TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_decode IN ");
	uint8_t F, P, sbit, ebit;
	const uint8_t* pdata = (const uint8_t*)in_data;
	const uint8_t* pay_ptr = tsk_null;
	tsk_size_t pay_size = 0;
	tsk_size_t hdr_size;
	tsk_size_t xsize, retsize = 0;
	int ret;

	tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;
	const trtp_rtp_header_t* rtp_hdr = (const trtp_rtp_header_t*)proto_hdr;
	tsk_bool_t is_idr = tsk_false;

	if(!h263 || !in_data || !in_size || !out_data || !h263->decoder.context){
		TSK_DEBUG_ERROR("Invalid parameter");
		return 0;
	}

	/*	RFC 2190
		get F and P bits, used to determine the header Mode (A, B or C)
		F: 1 bit
		The flag bit indicates the mode of the payload header. F=0, mode A;
		F=1, mode B or mode C depending on P bit defined below.
		P: 1 bit
		Optional PB-frames mode as defined by the H.263 [4]. "0" implies
		normal I or P frame, "1" PB-frames. When F=1, P also indicates modes:
		mode B if P=0, mode C if P=1.

		I:  1 bit.
	   Picture coding type, bit 9 in PTYPE defined by H.263[4], "0" is
	   intra-coded, "1" is inter-coded.
	*/
	F = *pdata >> 7;
	P = (*pdata >> 6) & 0x01;

	/* SBIT and EBIT */
	sbit = (*pdata >> 3) & 0x0F;
	ebit = (*pdata & 0x07);

	if(F == 0){
		/*	MODE A
			0                   1                   2                   3
			0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			|F|P|SBIT |EBIT | SRC |I|U|S|A|R      |DBQ| TRB |    TR         |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		*/
		hdr_size = H263_HEADER_MODE_A_SIZE;
		is_idr = (in_size >= 2) && !(pdata[1] & 0x10) /* I==1 */;
	}
	else if(P == 0){ // F=1 and P=0
		/* MODE B
			 0                   1                   2                   3
			0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			|F|P|SBIT |EBIT | SRC | QUANT   |  GOBN   |   MBA           |R  |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			|I|U|S|A| HMV1        | VMV1        | HMV2        | VMV2        |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		*/
		hdr_size = H263_HEADER_MODE_B_SIZE;
		is_idr = (in_size >= 5) && !(pdata[4] & 0x80) /* I==1 */;
	}
	else{ // F=1 and P=1
		/* MODE C
			 0                   1                   2                   3
			0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			|F|P|SBIT |EBIT | SRC | QUANT   |  GOBN   |   MBA           |R  |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			|I|U|S|A| HMV1        | VMV1        | HMV2        | VMV2        |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			| RR                                  |DBQ| TRB |    TR         |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		*/
		hdr_size = H263_HEADER_MODE_C_SIZE;
		is_idr = (in_size >= 5) && !(pdata[4] & 0x80) /* I==1 */;
	}

	/* Check size */
	if(in_size < hdr_size){
		TSK_DEBUG_ERROR("Too short");
		return 0;
	}

	pay_ptr = (pdata + hdr_size);
	pay_size = (in_size - hdr_size);
	xsize = avpicture_get_size(h263->decoder.context->pix_fmt, h263->decoder.context->width, h263->decoder.context->height);

	/* Packet lost? */
	if(h263->decoder.last_seq != (rtp_hdr->seq_num - 1) && h263->decoder.last_seq){
		if(h263->decoder.last_seq == rtp_hdr->seq_num){
			// Could happen on some stupid emulators
			//TSK_DEBUG_INFO("Packet duplicated, seq_num=%d", rtp_hdr->seq_num);
			return 0;
		}
		TSK_DEBUG_WARN("Packet lost, seq_num=%d", rtp_hdr->seq_num);
	}
	h263->decoder.last_seq = rtp_hdr->seq_num;

	if((int)(h263->decoder.accumulator_pos + pay_size) <= xsize){
		if((h263->decoder.ebit + sbit) == 8){ /* Perfect one Byte to clean up */
			if(h263->decoder.accumulator_pos){
				((uint8_t*)h263->decoder.accumulator)[h263->decoder.accumulator_pos-1] = (((uint8_t*)h263->decoder.accumulator)[h263->decoder.accumulator_pos-1] & (0xFF << h263->decoder.ebit)) |
					(*pay_ptr & (0xFF >> sbit));
			}
			pay_ptr++, pay_size--;
		}
		h263->decoder.ebit = ebit;

		memcpy(&((uint8_t*)h263->decoder.accumulator)[h263->decoder.accumulator_pos], pay_ptr, pay_size);
		h263->decoder.accumulator_pos += pay_size;
	}
	else{
		TSK_DEBUG_WARN("Buffer overflow");
		h263->decoder.accumulator_pos = 0;
		return 0;
	}

        // Check if decoder is in Pause state
        if (!h263->decoder.dec_started) {
            // Change state to Executing, if it was in Pause state
            StateTrans(h263, OMX_StateExecuting);
            h263->decoder.dec_started = tsk_true;
        }

	if(rtp_hdr->marker){
		AVPacket packet;
		/* allocate destination buffer */
		if(*out_max_size <xsize){
			if(!(*out_data = tsk_realloc(*out_data, xsize))){
				TSK_DEBUG_ERROR("Failed to allocate new buffer");
				h263->decoder.accumulator_pos = 0;
				*out_max_size = 0;
				return 0;
			}
			*out_max_size = xsize;
		}

        //TSK_DEBUG_INFO("in_size %d, out_max_size %d\n", in_size, *out_max_size);
		/* decode the picture */
		av_init_packet(&packet);
		packet.size = h263->decoder.accumulator_pos;
		packet.data = (uint8_t*)h263->decoder.accumulator;

                ret = hw_vpu_decode(h263, &packet);

		if(ret <0){
		        TSK_DEBUG_WARN("Failed to decode the buffer with error code =%d", ret);
			if(TMEDIA_CODEC_VIDEO(self)->in.callback){
				TMEDIA_CODEC_VIDEO(self)->in.result.type = tmedia_video_decode_result_type_error;
				TMEDIA_CODEC_VIDEO(self)->in.result.proto_hdr = proto_hdr;
				TMEDIA_CODEC_VIDEO(self)->in.callback(&TMEDIA_CODEC_VIDEO(self)->in.result);
			}
                } else if ((h263->decoder.out_ring_buf && h263->decoder.out_ring_buf->count > 0)) {
            tsk_size_t xsize;
            // Is it IDR frame?
            if(is_idr && TMEDIA_CODEC_VIDEO(self)->in.callback){
                TSK_DEBUG_INFO("Decoded H.263 IDR");
                TMEDIA_CODEC_VIDEO(self)->in.result.type = tmedia_video_decode_result_type_idr;
                TMEDIA_CODEC_VIDEO(self)->in.result.proto_hdr = proto_hdr;
                TMEDIA_CODEC_VIDEO(self)->in.callback(&TMEDIA_CODEC_VIDEO(self)->in.result);
            }
			/* fill out */
                        xsize = h263->decoder.decode_frame_size;

			if(*out_max_size<xsize){
                                //TSK_DEBUG_INFO("*out_max_size<xsize: *out_max_size %d, xsize %d\n", *out_max_size, xsize);
				if((*out_data = tsk_realloc(*out_data, (xsize + FF_INPUT_BUFFER_PADDING_SIZE)))){
					*out_max_size = xsize;
				}
				else{
					*out_max_size = 0;
					return 0;
				}
			}

			retsize = xsize;
                        copyDecodedData(h263, out_data, xsize);
			//TSK_DEBUG_INFO("hw vpu: retsize %d ", retsize);

                }
        /* in all cases: reset accumulator */
		h263->decoder.accumulator_pos = 0;
        } else if ((h263->decoder.out_ring_buf && h263->decoder.out_ring_buf->count > 0)) {
                // always check for data
                tsk_size_t xsize;
                    /* IDR ? */
                    if(((pay_ptr[0] & 0x1F) == 0x05) && TMEDIA_CODEC_VIDEO(self)->in.callback){
                            TSK_DEBUG_INFO("Decoded H.263 IDR");
                            TMEDIA_CODEC_VIDEO(self)->in.result.type = tmedia_video_decode_result_type_idr;
                            TMEDIA_CODEC_VIDEO(self)->in.result.proto_hdr = proto_hdr;
                            TMEDIA_CODEC_VIDEO(self)->in.callback(&TMEDIA_CODEC_VIDEO(self)->in.result);
                    }
                    /* fill out */
                    xsize = h263->decoder.decode_frame_size;

                    if(*out_max_size<xsize){
                        //TSK_DEBUG_INFO(" out_max_size < xsize: out_max_size %d, xsize %d\n", *out_max_size, xsize);
                        if((*out_data = tsk_realloc(*out_data, (xsize + FF_INPUT_BUFFER_PADDING_SIZE)))){
                            *out_max_size = xsize;
                        } else {
                            TSK_DEBUG_INFO("out_max_size<xsize: failed to realloc memeory");
                            *out_max_size = 0;
                            return 0;
                        }
                    }

                    retsize = xsize;
                    copyDecodedData(h263, out_data, xsize);
        }
        //TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_decode OUT ");
	return retsize;
}

static tsk_bool_t tdav_codec_h263_hw_vpu_sdp_att_match(const tmedia_codec_t* codec, const char* att_name, const char* att_value)
{
    if (!codec || !att_name || !att_value) {
        TSK_DEBUG_ERROR("Invalid parameter");
        return tsk_false;
    }

    if(tsk_striequals(att_name, "fmtp")){
        unsigned width, height, fps;
        if(tmedia_parse_video_fmtp(att_value, TMEDIA_CODEC_VIDEO(codec)->pref_size, &width, &height, &fps)){
            TSK_DEBUG_ERROR("Failed to match fmtp=%s", att_value);
            return tsk_false;
        }
        TMEDIA_CODEC_VIDEO(codec)->in.width = TMEDIA_CODEC_VIDEO(codec)->out.width = width;
        TMEDIA_CODEC_VIDEO(codec)->in.height = TMEDIA_CODEC_VIDEO(codec)->out.height = height;
        TMEDIA_CODEC_VIDEO(codec)->in.fps = TMEDIA_CODEC_VIDEO(codec)->out.fps = fps;
    }
    return tsk_true;
}

static char* tdav_codec_h263_hw_vpu_sdp_att_get(const tmedia_codec_t* codec, const char* att_name)
{
    if (!codec || !att_name) {
        TSK_DEBUG_ERROR("Invalid parameter");
        return tsk_false;
    }

    if(tsk_striequals(att_name, "fmtp")){
        tmedia_pref_video_size_t cif_vs;
        if(tmedia_video_get_closest_cif_size(TMEDIA_CODEC_VIDEO(codec)->pref_size, &cif_vs)){
            TSK_DEBUG_ERROR("Failed to get closest CIF family size");
            return tsk_null;
        }
        return tmedia_get_video_fmtp(cif_vs);
    }
    return tsk_null;
}

static int _tdav_session_h263_timer_cb(const void* arg, tsk_timer_id_t timer_id)
{
    int ret = 0;
    tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)arg;

    if (!h263) {
        TSK_DEBUG_ERROR("Invalid h263 parameter");
        return ret;
    }

    if (timer_id == h263->timer_idle.id) {
        // Change OMX Codec state to PAUSE if it is still not active after timout period
        if (!h263->encoder.enc_started) {
            TSK_DEBUG_INFO("Setting enoder OMX state to Pause");
            ret = EncStateTrans(h263, OMX_StatePause);
            if (ret != OMX_ErrorNone) {
                TSK_DEBUG_ERROR("Unable to do encoder state trans to Pause, ret %d", ret);
            }
        }
        if (!h263->decoder.dec_started) {
            TSK_DEBUG_INFO("Setting decoder OMX state to Pause");
            ret = StateTrans(h263, OMX_StatePause);
            if (ret != OMX_ErrorNone) {
                TSK_DEBUG_ERROR("Unable to do decoder state trans to Pause");
            }
        }

        // Stop timer
        if (h263->h_timer) {
            tsk_timer_manager_stop(h263->h_timer);
            h263->timer_started = tsk_false;
            h263->h_timer = tsk_null;
        }
    } else if (timer_id == h263->timer_reset_dec.id) {
        TSK_DEBUG_INFO("reset decoder timer");
        reset_decoder(h263);

        // Stop timer
        if (h263->h_timer) {
            tsk_timer_manager_stop(h263->h_timer);
            h263->timer_started = tsk_false;
            h263->h_timer = tsk_null;
        }
    } else {
        TSK_DEBUG_WARN("Invalid timer_id");
    }

    return ret;
}

/* constructor */
static tsk_object_t* tdav_codec_h263_hw_vpu_ctor(tsk_object_t * self, va_list * app)
{
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)self;
    if(h263){
        /* init base: called by tmedia_codec_create() */
        /* init self */
        tdav_codec_h263_init(TDAV_CODEC_H263(self), tdav_codec_h263_1996, CODEC_ID_H263, CODEC_ID_H263);
    }
    return self;
}

/* destructor */
static tsk_object_t* tdav_codec_h263_hw_vpu_dtor(tsk_object_t * self)
{
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)self;
    if(h263){
        /* deinit base */
        //tdav_codec_h263_common_deinit(TDAV_CODEC_H263_COMMON(self));
        tmedia_codec_video_deinit(h263);
        /* deinit self */
        tdav_codec_h263_deinit(h263);
    }

    return self;
}

/* object definition */
static const tsk_object_def_t tdav_codec_h263_hw_vpu_def_s =
{
        sizeof(tdav_codec_h263_hw_vpu_t),
        tdav_codec_h263_hw_vpu_ctor,
        tdav_codec_h263_hw_vpu_dtor,
        tmedia_codec_cmp,
};

/* plugin definition*/
static const tmedia_codec_plugin_def_t tdav_codec_h263_hw_vpu_plugin_def_s =
{
        &tdav_codec_h263_hw_vpu_def_s,

        tmedia_video,
        tmedia_codec_id_h263,
        "H263",
        "H263-1996 codec",
        TMEDIA_CODEC_FORMAT_H263,
        tsk_false,
        90000, // rate

        /* audio */
        { 0 },

        /* video */
        {176, 144, 15},

        tdav_codec_h263_hw_vpu_set,
        tdav_codec_h263_hw_vpu_open,
        tdav_codec_h263_hw_vpu_close,
        tdav_codec_h263_hw_vpu_encode,
        tdav_codec_h263_hw_vpu_decode,
        tdav_codec_h263_hw_vpu_sdp_att_match,
        tdav_codec_h263_hw_vpu_sdp_att_get
};
extern const tmedia_codec_plugin_def_t *tdav_codec_h263_hw_vpu_plugin_def_t = &tdav_codec_h263_hw_vpu_plugin_def_s;























/* ============ H.263-1998 Plugin interface ================= */

//
//	H.263-1998 object definition
//

// Not in use, as FSL doesn't support this profile. Use base profile for all profiels.
#if 0
static tsk_size_t tdav_codec_h263p_hw_vpu_decode(tmedia_codec_t* self, const void* in_data, tsk_size_t in_size, void** out_data, tsk_size_t* out_max_size, const tsk_object_t* proto_hdr)
{
	uint8_t P, V, PLEN, PEBIT;
	uint8_t* pdata = (uint8_t*)in_data;
	const uint8_t* pay_ptr;
	tsk_size_t pay_size;
	int hdr_size = H263P_HEADER_SIZE;
	tsk_size_t xsize, retsize = 0;
	int got_picture_ptr;
	int ret;

	tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;
	const trtp_rtp_header_t* rtp_hdr = (const trtp_rtp_header_t*)proto_hdr;

	if(!self || !in_data || !in_size || ((int)in_size <= hdr_size) || !out_data || !h263->decoder.context){
		TSK_DEBUG_ERROR("Invalid parameter");
		return 0;
	}

/*
	rfc4629 - 5.1.  General H.263+ Payload Header

         0                   1
         0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
        |   RR    |P|V|   PLEN    |PEBIT|
        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/
	P = (pdata[0] & 0x04)>>2;
	V = (pdata[0] & 0x02)>>1;
	PLEN = (((pdata[0] & 0x01)<<5) | pdata[1]>>3);
	PEBIT = pdata[1] & 0x07;

	if(V){
		/*
			Indicates the presence of an 8-bit field containing information
			for Video Redundancy Coding (VRC), which follows immediately after
			the initial 16 bits of the payload header, if present.  For syntax
			and semantics of that 8-bit VRC field, see Section 5.2.
		*/
	}
	if(PLEN){
			/*
				Length, in bytes, of the extra picture header.  If no extra
				picture header is attached, PLEN is 0.  If PLEN>0, the extra
				picture header is attached immediately following the rest of the
				payload header.  Note that the length reflects the omission of the
				first two bytes of the picture start code (PSC).  See Section 6.1.
			*/
		hdr_size += PLEN;
		if(PEBIT){
			/*
				Indicates the number of bits that shall be ignored in the last
				byte of the picture header.  If PLEN is not zero, the ignored bits
				shall be the least significant bits of the byte.  If PLEN is zero,
				then PEBIT shall also be zero.
			*/
			TSK_DEBUG_WARN("PEBIT ignored");
		}
	}
	if(P){ /* MUST be done after PLEN and PEBIT */
			/*
				Indicates the picture start or a picture segment (GOB/Slice) start
				or a video sequence end (EOS or EOSBS).  Two bytes of zero bits
				then have to be prefixed to the payload of such a packet to
				compose a complete picture/GOB/slice/EOS/EOSBS start code.  This
				bit allows the omission of the two first bytes of the start codes,
				thus improving the compression ratio.
			*/
		hdr_size -= 2;
		pdata[hdr_size] = 0x00, pdata[hdr_size + 1] = 0x00;
	}

	pay_ptr = (pdata + hdr_size);
	pay_size = (in_size - hdr_size);
	xsize = avpicture_get_size(h263->decoder.context->pix_fmt, h263->decoder.context->width, h263->decoder.context->height);

	/* Packet lost? */
	if(h263->decoder.last_seq != (rtp_hdr->seq_num - 1) && h263->decoder.last_seq){
		if(h263->decoder.last_seq == rtp_hdr->seq_num){
			// Could happen on some stupid emulators
			//TSK_DEBUG_INFO("Packet duplicated, seq_num=%d", rtp_hdr->seq_num);
			return 0;
		}
		TSK_DEBUG_INFO("Packet lost, seq_num=%d", rtp_hdr->seq_num);
	}
	h263->decoder.last_seq = rtp_hdr->seq_num;

	if((int)(h263->decoder.accumulator_pos + pay_size) <= xsize){
		/* PEBIT is ignored */
		memcpy(&((uint8_t*)h263->decoder.accumulator)[h263->decoder.accumulator_pos], pay_ptr, pay_size);
		h263->decoder.accumulator_pos += pay_size;
	}
	else{
		TSK_DEBUG_WARN("Buffer overflow");
		h263->decoder.accumulator_pos = 0;
		return 0;
	}

	if(rtp_hdr->marker){
		AVPacket packet;
		/* allocate destination buffer */
		if(*out_max_size < xsize){
			if(!(*out_data = tsk_realloc(*out_data, xsize))){
				TSK_DEBUG_ERROR("Failed to allocate new buffer");
				*out_max_size = 0;
				h263->decoder.accumulator_pos = 0;
				return 0;
			}
			*out_max_size = xsize;
		}

		/* decode the picture */
		av_init_packet(&packet);
		packet.size = h263->decoder.accumulator_pos;
		packet.data = (uint8_t*)h263->decoder.accumulator;
                ret = hw_vpu_decode(h263, h263->decoder.decode_buffer, &got_picture_ptr, &packet);

		if(ret <0 || !got_picture_ptr){
			TSK_DEBUG_WARN("Failed to decode the buffer");
		}
		else{
		    xsize = avpicture_get_size(h263->decoder.context->pix_fmt, h263->decoder.context->width, h263->decoder.context->height);

			if(*out_max_size<xsize){
                //TSK_DEBUG_INFO("*out_max_size<xsize: *out_max_size %d, xsize %d\n", *out_max_size, xsize);
				if((*out_data = tsk_realloc(*out_data, (xsize + FF_INPUT_BUFFER_PADDING_SIZE)))){
					*out_max_size = xsize;
				}
				else{
					*out_max_size = 0;
					return 0;
				}
			}

		    retsize = xsize;
		    memcpy(*out_data, h263->decoder.decode_buffer, xsize);
		    //TSK_DEBUG_INFO("hw vpu: retsize %d ", retsize);



		}
		/* in all cases: reset accumulator */
		h263->decoder.accumulator_pos = 0;
	}

	return retsize;
}
#endif

/* constructor */
static tsk_object_t* tdav_codec_h263p_hw_vpu_ctor(tsk_object_t * self, va_list * app)
{
	tdav_codec_h263p_hw_vpu_t *h263p = (tdav_codec_h263p_hw_vpu_t*)self;
	if(h263p){
		/* init base: called by tmedia_codec_create() */
		/* init self */
		tdav_codec_h263_init(TDAV_CODEC_H263(self), tdav_codec_h263_1998, CODEC_ID_H263P, CODEC_ID_H263);
	}
	return self;
}
/* destructor */
static tsk_object_t* tdav_codec_h263p_hw_vpu_dtor(tsk_object_t * self)
{
    TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_main_dtor IN \n");
	tdav_codec_h263p_hw_vpu_t *h263p = (tdav_codec_h263p_hw_vpu_t*)self;
	if(h263p){
		/* deinit base */
		tmedia_codec_video_deinit(h263p);
		/* deinit self */
		tdav_codec_h263_deinit(TDAV_CODEC_H263(self));
	}

    TSK_DEBUG_INFO("tdav_codec_h263_hw_vpu_main_dtor OUT \n");
	return self;
}

/* object definition */
static const tsk_object_def_t tdav_codec_h263p_hw_vpu_def_s =
{
	sizeof(tdav_codec_h263p_hw_vpu_t),
	tdav_codec_h263p_hw_vpu_ctor,
	tdav_codec_h263p_hw_vpu_dtor,
	tmedia_codec_cmp,
};
/* plugin definition*/
static const tmedia_codec_plugin_def_t tdav_codec_h263p_hw_vpu_plugin_def_s =
{
	&tdav_codec_h263p_hw_vpu_def_s,

	tmedia_video,
	tmedia_codec_id_h263p,
	"H263-1998",
	"H263-1998 codec",
	TMEDIA_CODEC_FORMAT_H263_1998,
	tsk_true,
	90000, // rate

	/* audio */
	{ 0 },

	/* video */
	{176, 144, 15},

	tdav_codec_h263p_hw_vpu_set,
	tdav_codec_h263p_hw_vpu_open,
	tdav_codec_h263p_hw_vpu_close,
	tdav_codec_h263p_hw_vpu_encode,
	tdav_codec_h263p_hw_vpu_decode,
	tdav_codec_h263p_hw_vpu_sdp_att_match,
	tdav_codec_h263p_hw_vpu_sdp_att_get
};
extern const tmedia_codec_plugin_def_t *tdav_codec_h263p_hw_vpu_plugin_def_t = &tdav_codec_h263p_hw_vpu_plugin_def_s;














/* ============ H.263-2000 Plugin interface ================= */

//
//	H.263-2000 object definition
//

/* constructor */
static tsk_object_t* tdav_codec_h263pp_hw_vpu_ctor(tsk_object_t * self, va_list * app)
{
	tdav_codec_h263pp_hw_vpu_t *h263pp = (tdav_codec_h263pp_hw_vpu_t*) self;
	if(h263pp){
		/* init base: called by tmedia_codec_create() */
		/* init self */
		tdav_codec_h263_init(TDAV_CODEC_H263(self), tdav_codec_h263_2000, CODEC_ID_H263P, CODEC_ID_H263);
	}
	return self;
}
/* destructor */
static tsk_object_t* tdav_codec_h263pp_hw_vpu_dtor(tsk_object_t * self)
{
	tdav_codec_h263pp_hw_vpu_t *h263pp = (tdav_codec_h263pp_hw_vpu_t*) self;
	if(h263pp){
		/* deinit base */
		tmedia_codec_video_deinit(h263pp);
		/* deinit self */
		tdav_codec_h263_deinit(TDAV_CODEC_H263(self));
	}

	return self;
}
/* object definition */
static const tsk_object_def_t tdav_codec_h263pp_hw_vpu_def_s =
{
	sizeof(tdav_codec_h263pp_hw_vpu_t),
	tdav_codec_h263pp_hw_vpu_ctor,
	tdav_codec_h263pp_hw_vpu_dtor,
	tmedia_codec_cmp,
};
/* plugin definition*/
static const tmedia_codec_plugin_def_t tdav_codec_h263pp_hw_vpu_plugin_def_s =
{
	&tdav_codec_h263pp_hw_vpu_def_s,

	tmedia_video,
	tmedia_codec_id_h263pp,
	"H263-2000",
	"H263-2000 codec",
	TMEDIA_CODEC_FORMAT_H263_2000,
	tsk_true,
	90000, // rate

	/* audio */
	{ 0 },

	/* video */
	{176, 144, 15},

	tdav_codec_h263pp_hw_vpu_set,
	tdav_codec_h263pp_hw_vpu_open,
	tdav_codec_h263pp_hw_vpu_close,
	tdav_codec_h263pp_hw_vpu_encode,
	tdav_codec_h263pp_hw_vpu_decode,
	tdav_codec_h263pp_hw_vpu_sdp_att_match,
	tdav_codec_h263pp_hw_vpu_sdp_att_get
};
extern const tmedia_codec_plugin_def_t *tdav_codec_h263pp_hw_vpu_plugin_def_t = &tdav_codec_h263pp_hw_vpu_plugin_def_s;



int tdav_codec_h263_open_encoder(tdav_codec_h263_hw_vpu_t* self)
{
    if (!self) {
        TSK_DEBUG_ERROR("Invalid parameter");
        return -1;
    }

    int ret;
    int size;
    if(self->encoder.context || self->encoder.hComponent){
        TSK_DEBUG_ERROR("Encoder already opened");
        return -1;
    }

    self->encoder.context = avcodec_alloc_context();
    if (!self->encoder.context) {
        TSK_DEBUG_ERROR("Failed to allocate context");
        return -1;
    }

    avcodec_get_context_defaults(self->encoder.context);

    self->encoder.context->pix_fmt		= PIX_FMT_YUV420P;
    self->encoder.context->time_base.num  = TIME_BASE_NUM;
    self->encoder.context->time_base.den  = TMEDIA_CODEC_VIDEO(self)->out.fps;
    self->encoder.context->width = TMEDIA_CODEC_VIDEO(self)->out.width;
    self->encoder.context->height = TMEDIA_CODEC_VIDEO(self)->out.height;

    self->encoder.context->qmin = 10;
    self->encoder.context->qmax = 51;
#if LIBAVCODEC_VERSION_MAJOR <= 53
    self->encoder.context->mb_qmin = self->encoder.context->qmin;
    self->encoder.context->mb_qmax = self->encoder.context->qmax;
#endif
    self->encoder.context->mb_decision = FF_MB_DECISION_RD;

    self->encoder.context->bit_rate = ((TMEDIA_CODEC_VIDEO(self)->out.width *
        TMEDIA_CODEC_VIDEO(self)->out.height * BIT_RATE_CAL_PARAM1 / BIT_RATE_CAL_PARAM2 / BIT_RATE_CAL_PARAM3) * MUL_FACTOR);
    //self->encoder.context->rc_lookahead = 0;
    self->encoder.context->rtp_payload_size = RTP_PAYLOAD_SIZE;
    self->encoder.context->opaque = tsk_null;
    self->encoder.context->gop_size = (TMEDIA_CODEC_VIDEO(self)->out.fps * TDAV_H263_GOP_SIZE_IN_SECONDS);
    self->encoder.context->flags |= CODEC_FLAG_QSCALE;
    self->encoder.context->global_quality = FF_QP2LAMBDA * self->encoder.quality;
    self->encoder.context->max_b_frames = 0;

    size = avpicture_get_size(PIX_FMT_YUV420P, self->encoder.context->width, self->encoder.context->height);

	// RTP Callback
	switch(self->type){
		case tdav_codec_h263_1996:
			{	// H263 - 1996
				break;
			}
		case tdav_codec_h263_1998:
			{	// H263 - 1998
#if defined(CODEC_FLAG_H263P_UMV)
				self->encoder.context->flags |= CODEC_FLAG_H263P_UMV;		// Annex D+
#endif
				self->encoder.context->flags |= CODEC_FLAG_AC_PRED;			// Annex I and T
				self->encoder.context->flags |= CODEC_FLAG_LOOP_FILTER;		// Annex J
#if defined(CODEC_FLAG_H263P_SLICE_STRUCT)
				self->encoder.context->flags |= CODEC_FLAG_H263P_SLICE_STRUCT;	// Annex K
#endif
#if defined(CODEC_FLAG_H263P_AIV)
				self->encoder.context->flags |= CODEC_FLAG_H263P_AIV;			// Annex S
#endif
				break;
			}
		case tdav_codec_h263_2000:
			{	// H263 - 2000
#if defined(CODEC_FLAG_H263P_UMV)
				self->encoder.context->flags |= CODEC_FLAG_H263P_UMV;		// Annex D+
#endif
				self->encoder.context->flags |= CODEC_FLAG_AC_PRED;			// Annex I and T
				self->encoder.context->flags |= CODEC_FLAG_LOOP_FILTER;		// Annex J
#if defined(CODEC_FLAG_H263P_SLICE_STRUCT)
				self->encoder.context->flags |= CODEC_FLAG_H263P_SLICE_STRUCT;	// Annex K
#endif
#if defined(CODEC_FLAG_H263P_AIV)
				self->encoder.context->flags |= CODEC_FLAG_H263P_AIV;			// Annex S
#endif
				break;
			}
	}

    self->encoder.decoded_first_frame = OMX_FALSE;
    self->encoder.encoded_first_frame = OMX_FALSE;

    if (!set_bitrate_fac_GOP_size(self)) {
        return -1;
    }

    TSK_DEBUG_INFO("open_encoder: bitrate factor %d, gop %d",
                    self->encoder.bitrate_fact, self->encoder.gop_size);

    // Open hw_vpu encoder
    ret = hw_vpu_open_encoder(self);

    if (ret < 0) {
        TSK_DEBUG_ERROR("Error opening encoder, ret code %d", ret);
        return -1;
    }

    if (!self->disable_dummy_buffer) {
        // Send width and height without alignment to match OMX buf len.
        OMX_S32 width = TMEDIA_CODEC_VIDEO(self)->in.width;
        OMX_S32 height = TMEDIA_CODEC_VIDEO(self)->in.height;

        size = width * height * 3 / 2;
        if ((self->encoder.dummy_decode_buffer = tsk_calloc(size, sizeof(uint8_t)))) {
            self->encoder.dummy_decode_buffer_size = size;
            //start encoding asap with dummy buffer
            /* decode the picture */
            AVPacket packet;
            av_init_packet(&packet);
            packet.size = self->encoder.dummy_decode_buffer_size;
            packet.data = (uint8_t*)self->encoder.dummy_decode_buffer;
            TSK_DEBUG_INFO("Sending dummy decode buffer at encoder open");
            hw_vpu_encode(self, &packet);
        }
    }
    return ret;
}

int tdav_codec_h263_open_decoder(tdav_codec_h263_hw_vpu_t* self)
{

    //TSK_DEBUG_INFO("tdav_codec_h263_open_decoder in\n");
	int ret, size;

	if(self->decoder.context || self->decoder.hComponent){
		TSK_DEBUG_ERROR("Decoder already opened");
		return -1;
	}

	self->decoder.context = avcodec_alloc_context();
	avcodec_get_context_defaults(self->decoder.context);

	self->decoder.context->pix_fmt = PIX_FMT_YUV420P;
	self->decoder.context->width = Align(TMEDIA_CODEC_VIDEO(self)->in.width, FRAME_ALIGN); //TMEDIA_CODEC_VIDEO(self)->in.width;
	self->decoder.context->height = Align(TMEDIA_CODEC_VIDEO(self)->in.height, FRAME_ALIGN); //TMEDIA_CODEC_VIDEO(self)->in.height;
	size = avpicture_get_size(PIX_FMT_YUV420P, self->decoder.context->width, self->decoder.context->height);
	if(!(self->decoder.accumulator = tsk_calloc((size + FF_INPUT_BUFFER_PADDING_SIZE), sizeof(uint8_t)))){
		TSK_DEBUG_ERROR("Failed to allocate decoder buffer");
		return -2;
	}

        self->decoder.decode_frame_size = self->decoder.context->width * self->decoder.context->height * 3 / 2;
        TSK_DEBUG_INFO("open_decoder width, height (%d, %d),dec size %d\n",
                             self->decoder.context->width, self->decoder.context->height, self->decoder.decode_frame_size);

        if (!(self->decoder.crop_buffer = tsk_calloc(self->decoder.decode_frame_size, sizeof(uint8_t)))) {
                TSK_DEBUG_ERROR("Failed to allocate decoder crop buffer");
                return -2;
        }

        // Open hw_vpu decoder
        ret = hw_vpu_open_decoder(self);

        if (ret < 0) {
            TSK_DEBUG_ERROR("Error opening decoder, ret code %d", ret);
            return -1;
        }

        if (!self->disable_dummy_buffer) {
            OMX_S32 frame_len = 0;
            int height = TMEDIA_CODEC_VIDEO(self)->in.height;
            const char* path = tsk_null;

            if (height == HT_720P) {
                path = DUMMY_720P_FRAME_PATH;
            } else if (height == HT_VGA) {
                path = DUMMY_VGA_FRAME_PATH;
            } else if (height == HT_QVGA) {
                path = DUMMY_QVGA_FRAME_PATH;
            }

            TSK_DEBUG_INFO("open_decoder: File path %s", path);
            if (path != tsk_null) {
                FILE *fp = fopen(path, "rb");
                if (fp != NULL) {
                    fseek(fp, 0L, SEEK_END);
                    frame_len = ftell(fp);
                    fseek(fp, 0L, SEEK_SET);
                    //TSK_DEBUG_INFO("frame_len %d", frame_len);
                    if(!(self->encoder.dummy_buffer = tsk_calloc(frame_len, sizeof(uint8_t)))){
                        TSK_DEBUG_ERROR("Failed to allocate encoder buffer, can be ignred");
                        fclose(fp);
                        return ret;
                    }
                    self->encoder.dummy_buffer_size = fread(self->encoder.dummy_buffer, 1, frame_len, fp);
                    fclose(fp);

                    AVPacket packet;
                    /* decode the picture */
                    av_init_packet(&packet);
                    packet.size = self->encoder.dummy_buffer_size;
                    packet.data = (uint8_t*)self->encoder.dummy_buffer;

                    TSK_DEBUG_INFO("sending dummy encoded frame with size %d", self->encoder.dummy_buffer_size);
                    ret = hw_vpu_decode(self, &packet);
                } else {
                    TSK_DEBUG_ERROR("Failed to open file at %s with %d", path, errno);
                }
            }
        }

        TSK_DEBUG_INFO("open_decoder out\n");

	return ret;
}

int tdav_codec_h263_close_encoder(tdav_codec_h263_hw_vpu_t* self)
{
    if(self->encoder.context){
        //avcodec_close(self->encoder.context);
        av_free(self->encoder.context);
        self->encoder.context = tsk_null;
    }

    // close vpu encoder
    if (self->encoder.hComponent) {
        close_hw_vpu_encoder(self);
        self->encoder.hComponent = NULL;
    }

    return 0;
}

int tdav_codec_h263_close_decoder(tdav_codec_h263_hw_vpu_t* self)
{
    TSK_DEBUG_INFO("tdav_codec_h263_close_decoder IN \n");
    if(self->decoder.context){
        //avcodec_close(self->decoder.context);
        av_free(self->decoder.context);
        self->decoder.context = tsk_null;
    }

    if (self->decoder.hComponent) {
        close_hw_vpu_decoder(self);
        self->decoder.hComponent = NULL;
    }

    if(self->decoder.accumulator){
        TSK_FREE(self->decoder.accumulator);
        self->decoder.accumulator_pos = 0;
    }

    if(self->decoder.crop_buffer){
        TSK_FREE(self->decoder.crop_buffer);
    }

    TSK_DEBUG_INFO("tdav_codec_h263_close_decoder OUT \n");
    return 0;
}

/* ============ Callbacks ================= */

static void tdav_codec_h263_encap(const tdav_codec_h263_hw_vpu_t* h263, const uint8_t* pdata, tsk_size_t size)
{
#if 0
    /*Default Doubango RTP Encapsulation error due to following code - this expects only ONE RTP packet per Video Frame*/
	tsk_bool_t frag = tsk_false;
	register uint32_t i, last_index = 0;

	if(size < RTP_PAYLOAD_SIZE){
		goto last;
	}

	for(i = 4; i<(size - 4); i++){
		if(pdata[i] == 0x00 && pdata[i+1] == 0x00 && pdata[i+2]>=0x80){  /* PSC or (GBSC) found */
			if((i - last_index) >= RTP_PAYLOAD_SIZE || tsk_true/* FIXME */){
				switch(h263->type){
					case tdav_codec_h263_1996:
						tdav_codec_h263_rtp_callback((tdav_codec_h263_hw_vpu_t*) h263, pdata+last_index,
							(i - last_index), (last_index == size));
						break;
					default:
						tdav_codec_h263p_rtp_callback((tdav_codec_h263_hw_vpu_t*) h263, pdata + last_index,
							(i - last_index), frag, (last_index == size));
						frag = tsk_true;
						break;
				}
				last_index = i;
			}
		}
	}
last:
	if(last_index < size){
		switch(h263->type){
			case tdav_codec_h263_1996:
				tdav_codec_h263_rtp_callback((tdav_codec_h263_hw_vpu_t*) h263, pdata + last_index,
					(size - last_index), tsk_true);
				break;
			default:
				tdav_codec_h263p_rtp_callback((tdav_codec_h263_hw_vpu_t*) h263, pdata + last_index,
					(size - last_index), frag, tsk_true);
				break;
		}
	}
#endif

    register uint32_t i;

    if(pdata[0] == 0x00 && pdata[1] == 0x00 && (pdata[2] & 0xfc)==0x80) { /* PSC or (GBSC) found */
        for(i = 0; i < size; i += (RTP_PAYLOAD_SIZE - RTP_PAYLOAD_CORRECTION)) {
            tdav_codec_h263_rtp_callback((tdav_codec_h263_hw_vpu_t*) h263, pdata + i,
					    ((size - i) >= (RTP_PAYLOAD_SIZE - RTP_PAYLOAD_CORRECTION)) ? (RTP_PAYLOAD_SIZE - RTP_PAYLOAD_CORRECTION) : (size - i), !((size - i) > (RTP_PAYLOAD_SIZE - RTP_PAYLOAD_CORRECTION)));
        }
    }
}


static void tdav_codec_h263_rtp_callback(tdav_codec_h263_hw_vpu_t *self, const void *data, tsk_size_t size, tsk_bool_t marker)
{
	uint8_t* pdata = (uint8_t*)data;

	if(self->rtp.size < (size + H263_HEADER_MODE_A_SIZE)){
		if(!(self->rtp.ptr = (uint8_t*)tsk_realloc(self->rtp.ptr, (size + H263_HEADER_MODE_A_SIZE)))){
			TSK_DEBUG_ERROR("Failed to allocate new buffer");
			return;
		}
		self->rtp.size = (size + H263_HEADER_MODE_A_SIZE);
	}
	memcpy((self->rtp.ptr + H263_HEADER_MODE_A_SIZE), data, size);

	/* http://eu.sabotage.org/www/ITU/H/H0263e.pdf section 5.1
	* 5.1.1 Picture Start Code (PSC) (22 bits) - PSC is a word of 22 bits. Its value is 0000 0000 0000 0000 1 00000.

	*
	* 5.1.1 Picture Start Code (PSC) (22 bits)
	* 5.1.2 Temporal Reference (TR) (8 bits)
	* 5.1.3 Type Information (PTYPE) (Variable Length)
	*	– Bit 1: Always "1", in order to avoid start code emulation.
	*	– Bit 2: Always "0", for distinction with Recommendation H.261.

	*	– Bit 3: Split screen indicator, "0" off, "1" on.
	*	– Bit 4: Document camera indicator, "0" off, "1" on.
	*	– Bit 5: Full Picture Freeze Release, "0" off, "1" on.
	*	– Bits 6-8: Source Format, "000" forbidden, "001" sub-QCIF, "010" QCIF, "011" CIF,
		"100" 4CIF, "101" 16CIF, "110" reserved, "111" extended PTYPE.
		If bits 6-8 are not equal to "111", which indicates an extended PTYPE (PLUSPTYPE), the following
		five bits are also present in PTYPE:
		– Bit 9: Picture Coding Type, "0" INTRA (I-picture), "1" INTER (P-picture).
		– Bit 10: Optional Unrestricted Motion Vector mode (see Annex D), "0" off, "1" on.
		– Bit 11: Optional Syntax-based Arithmetic Coding mode (see Annex E), "0" off, "1" on.
		– Bit 12: Optional Advanced Prediction mode (see Annex F), "0" off, "1" on.
		– Bit 13: Optional PB-frames mode (see Annex G), "0" normal I- or P-picture, "1" PB-frame.
	*/
	if(pdata[0] == 0x00 && pdata[1] == 0x00 && (pdata[2] & 0xfc)==0x80){ /* PSC */
		/* RFC 2190 -5.1 Mode A
			0                   1                   2                   3
			0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
			|F|P|SBIT |EBIT | SRC |I|U|S|A|R      |DBQ| TRB |    TR         |
			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

		SRC : 3 bits
		Source format, bit 6,7 and 8 in PTYPE defined by H.263 [4], specifies
		the resolution of the current picture.

		I:  1 bit.
		Picture coding type, bit 9 in PTYPE defined by H.263[4], "0" is
		intra-coded, "1" is inter-coded.
		*/

		// PDATA[4] ======> Bits 3-10 of PTYPE
		uint32_t rtp_hdr = 0;
		uint8_t format, pict_type;

		// Source Format = 4,5,6
		format = (pdata[4] & 0x3C)>>2;
		// Picture Coding Type = 7
		pict_type = (pdata[4] & 0x02)>>1;
		// RTP mode A header
		((uint8_t*)&rtp_hdr)[1] = (format <<5) | (pict_type << 4);
		//rtp_hdr = tnet_htonl(rtp_hdr);
		memcpy(self->rtp.ptr, &rtp_hdr, sizeof(rtp_hdr));
	}

	// Send data over the network
	if(TMEDIA_CODEC_VIDEO(self)->out.callback){
		TMEDIA_CODEC_VIDEO(self)->out.result.buffer.ptr = self->rtp.ptr;
		TMEDIA_CODEC_VIDEO(self)->out.result.buffer.size = (size + H263_HEADER_MODE_A_SIZE);
		TMEDIA_CODEC_VIDEO(self)->out.result.duration =  (uint32_t)((1./(double)TMEDIA_CODEC_VIDEO(self)->out.fps) * TMEDIA_CODEC(self)->plugin->rate);
		TMEDIA_CODEC_VIDEO(self)->out.result.last_chunck = marker;
		TMEDIA_CODEC_VIDEO(self)->out.callback(&TMEDIA_CODEC_VIDEO(self)->out.result);
	}
}

static void tdav_codec_h263p_rtp_callback(tdav_codec_h263_hw_vpu_t *self, const void *data, tsk_size_t size, tsk_bool_t frag, tsk_bool_t marker)
{
	uint8_t* pdata = (uint8_t*)data;
	//uint8_t rtp_hdr[2] = {0x00, 0x00};
	//tsk_bool_t eos = tsk_false;

	const void* _ptr = tsk_null;
	tsk_size_t _size = 0;
	//static tsk_bool_t frag = tsk_false;
	//tsk_bool_t found_gob = tsk_false;

	/* RFC 4629 - 5.1. General H.263+ Payload Header
		The H.263+ payload header is structured as follows:
         0                   1
         0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
        |   RR    |P|V|   PLEN    |PEBIT|
        +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	*/

	/* http://eu.sabotage.org/www/ITU/H/H0263e.pdf
	*
	* 5.1.1 Picture Start Code (PSC) (22 bits)
	*	->PSC is a word of 22 bits. Its value is 0000 0000 0000 0000 1 00000.
	* 5.1.27 End Of Sequence (EOS) (22 bits)
	*	->A codeword of 22 bits. Its value is 0000 0000 0000 0000 1 11111
	* 5.2.2 Group of Block Start Code (GBSC) (17 bits)
	*	->A word of 17 bits. Its value is 0000 0000 0000 0000 1
	* C.4.1 End Of Sub-Bitstream code (EOSBS) (23 bits)
	*	->The EOSBS code is a codeword of 23 bits. Its value is 0000 0000 0000 0000 1 11110 0
	*
	*
	* 5.2.3 Group Number (GN) (5 bits)
	*	-> last 5 bits
	*/
	//if(pdata[0] == 0x00 && pdata[1] == 0x00 && pdata[2] >= 0x80){ /* PSC or EOS or GBSC */
	//	uint8_t GN = ((pdata[2]>>2) & 0x1F);
	//	found_gob = tsk_true;
	//	//TSK_DEBUG_INFO("GN=%u", pdata[2]);
	//
	//	/*	RFC 4629 - 6.1.1. Packets that begin with a Picture Start Code
	//		A packet that begins at the location of a Picture, GOB, slice, EOS,
	//		or EOSBS start code shall omit the first two (all zero) bytes from
	//		the H.263+ bitstream and signify their presence by setting P=1 in the
	//		payload header.
	//	*/

	//	if(GN == 0x00){ /* PSC 00000 */
	//		/* Use the two first bytes as RTP header */
	//		//pdata[0] |= 0x04; // P=1

	//		/*
	//		 0                   1                   2                   3
	//		0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
	//		+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//		|   RR    |1|V|0|0|0|0|0|0|0|0|0| bitstream data without the    :
	//		+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//		: first two 0 bytes of the PSC
	//		+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//		*/

	//		//TSK_DEBUG_INFO("H263 - PSC");
	//	}
	//	else if(GN == 0x1F){ /* EOS 11111 */
	//		/* Use the two first bytes as RTP header */
	//		//pdata[0] |= 0x04; // P=1
	//		eos = tsk_true;
	//		/* RFC 4629 - 6.1.3. Packets that begin with an EOS or EOSBS Code
	//			0                   1                   2
	//			0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3
	//			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//			|   RR    |1|V|0|0|0|0|0|0|0|0|0|1|1|1|1|1|1|0|0|
	//			+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//		*/
	//		//TSK_DEBUG_INFO("H263 - EOS");
	//	}
	//	else /*if((GN >> 4) == 0x01)*/{ /* GBSC  10000 */
	//		/* Use the two first bytes as RTP header */
	//		//pdata[0] |= 0x04; // P=1
	//
	//		/* RFC 4629 - 6.1.2. Packets that begin with GBSC or SSC
	//	0                   1                   2                   3
	//	0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
	//	+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//	|   RR    |1|V|0 0 1 0 0 1|PEBIT|1 0 0 0 0 0| picture header    :
	//	+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//	: starting with TR, PTYPE ...                                   |
	//	+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//	| ...                                           | bitstream     :
	//	+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//	: data starting with GBSC/SSC without its first two 0 bytes
	//	+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
	//		*/
	//		//TSK_DEBUG_INFO("H263 - GBSC");
	//		found_gob = tsk_false;
	//	}
	//	//else if(EOSBS) -> Not Supported
	//}
	//else{
	//	/* 6.2. Encapsulating Follow-on Packet (P=0) */
	//	int i = 0;
	//	i++;
	//}

	//if(/*eos*/!found_gob && frag){
	//	if(self->rtp.size < (size + 2/* H263+ Header size */)){
	//		if(!(self->rtp.ptr = tsk_realloc(self->rtp.ptr, (size + 2)))){
	//			TSK_DEBUG_ERROR("Failed to allocate new buffer");
	//			return;
	//		}
	//		self->rtp.size = (size + 2);
	//	}
	//	/* RFC 4629 - 6. Packetization Schemes */
	//	//rtp_hdr[0] |= 0x00;
	//	//memcpy(self->rtp.ptr, rtp_hdr/* zeros-> is it corretc? */, 2);
	//	//memcpy((self->rtp.ptr + 2), pdata, size);
	//	//_ptr = self->rtp.ptr;
	//	//_size = (size + 2);

	//	pdata[0] |= pdata[2] > 0x80 ? 0x04 : 0x04;
	//	_ptr = pdata;
	//	_size = size;
	//}
	//else{
	//	pdata[0] |= pdata[2] > 0x80 ? 0x04 : 0x04;
	//	_ptr = pdata;
	//	_size = size;
	//}

// FIXME
	pdata[0] |= pdata[2] > 0x80 ? 0x04 : 0x04;
	_ptr = pdata;
	_size = size;


	// Send data over the network
	if(TMEDIA_CODEC_VIDEO(self)->out.callback){
		TMEDIA_CODEC_VIDEO(self)->out.result.buffer.ptr = _ptr;
		TMEDIA_CODEC_VIDEO(self)->out.result.buffer.size = _size;
		TMEDIA_CODEC_VIDEO(self)->out.result.duration =  (uint32_t)((1./(double)TMEDIA_CODEC_VIDEO(self)->out.fps) * TMEDIA_CODEC(self)->plugin->rate);
		TMEDIA_CODEC_VIDEO(self)->out.result.last_chunck = marker;
		TMEDIA_CODEC_VIDEO(self)->out.callback(&TMEDIA_CODEC_VIDEO(self)->out.result);
	}
}

//OMX code

#define NALU_START_CODE 0x01

OMX_ERRORTYPE read_enc_frame(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr)
{
    //TSK_DEBUG_INFO("read_enc_frame: input q size %d\n", h263->encoder.pInputMsgQ->Size());
    OMX_U32 size = 0;
    OMX_S32 frame_len = 0;

    if ((!h263 || !h263->encoder.in_ring_buf) || (!pBufferHdr || !pBufferHdr->pBuffer)) {
        TSK_DEBUG_ERROR("read_enc_frame: invalid parameter");
        return OMX_ErrorUndefined;
    }

    if (!h263->encoder.bThreadRunning) {
        TSK_DEBUG_ERROR("read_enc_frame: thread exited\n");
        return OMX_ErrorNone;
    }

    if (h263->encoder.in_ring_buf->count > 0) {
        int size = pBufferHdr->nAllocLen;
        bool realloc = tsk_false;// Don't allow to re-alloc OMX buffer
        ring_buffer_pop(h263->encoder.in_ring_buf, (void*) pBufferHdr->pBuffer, &size, realloc);
        if (size > 0) {
            pBufferHdr->nFilledLen = size;
            pBufferHdr->nFlags = OMX_BUFFERFLAG_ENDOFFRAME;
        } else {
            pBufferHdr->nFlags = OMX_BUFFERFLAG_EOS;
            h263->encoder.bInEos = OMX_TRUE;
            TSK_DEBUG_INFO("Input EOS.\n");
        }
        //TSK_DEBUG_INFO("read_enc_frame: got frame with size %d after %d tries",
        //                pBufferHdr->nFilledLen, h263->encoder.inBufEmptyCount);
        h263->encoder.inBufEmptyCount = 0;

    } else if (h263->encoder.encoded_first_frame == OMX_FALSE) {
        if (h263->encoder.dummy_decode_buffer != NULL && h263->encoder.dummy_decode_buffer_size > 0) {
            TSK_DEBUG_INFO("read_enc_frame: sending same frame");
            memcpy(pBufferHdr->pBuffer, h263->encoder.dummy_decode_buffer, h263->encoder.dummy_decode_buffer_size);

            pBufferHdr->nFilledLen = h263->encoder.dummy_decode_buffer_size;
            pBufferHdr->nFlags = OMX_BUFFERFLAG_ENDOFFRAME;
        }
    } else {
        if (h263->encoder.dummy_decode_buffer) {
            TSK_FREE(h263->encoder.dummy_decode_buffer);
        }

        //TSK_DEBUG_INFO("read_enc_frame: input msg Q is empty\n");
        pBufferHdr->nFilledLen = 0;
        h263->encoder.inBufEmptyCount++;
    }

    return OMX_ErrorNone;
}

OMX_ERRORTYPE read_frame(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr)
{
    OMX_U32 size = 0;
    OMX_S32 frame_len = 0;

    if ((!h263 || !h263->decoder.in_ring_buf) || (!pBufferHdr || !pBufferHdr->pBuffer)) {
        TSK_DEBUG_ERROR("read_frame: invalid parameter");
        return OMX_ErrorUndefined;
    }

    if (!h263->decoder.bThreadRunning) {
        TSK_DEBUG_ERROR("read_frame: thread exited\n");
        return OMX_ErrorNone;
    }

    pBufferHdr->nFilledLen = 0;
    pBufferHdr->nOffset = 0;

    if (h263->decoder.in_ring_buf->count > 0) {
        int size = pBufferHdr->nAllocLen;
        bool realloc = tsk_false;// Don't allow to re-alloc OMX buffer

        ring_buffer_pop(h263->decoder.in_ring_buf, (void*) pBufferHdr->pBuffer, &size, realloc);
        if (size > 0) {
            pBufferHdr->nFilledLen = size;
            pBufferHdr->nFlags = OMX_BUFFERFLAG_ENDOFFRAME;
        } else {
            pBufferHdr->nFlags = OMX_BUFFERFLAG_EOS;
            h263->decoder.bInEos = OMX_TRUE;
            TSK_DEBUG_WARN("Input EOS.\n");
        }
        //TSK_DEBUG_INFO("read_frame: got frame with size %d after %d tries, rb count %d",
        //                pBufferHdr->nFilledLen, h263->decoder.inBufEmptyCount, h263->decoder.in_ring_buf->count);
        h263->decoder.inBufEmptyCount = 0;

    } else if (h263->encoder.decoded_first_frame == OMX_FALSE) {
        if (h263->encoder.dummy_buffer != NULL && h263->encoder.dummy_buffer_size > 0) {
            TSK_DEBUG_INFO("read_frame: sending same frame");
            memcpy(pBufferHdr->pBuffer, h263->encoder.dummy_buffer, h263->encoder.dummy_buffer_size);

            pBufferHdr->nFilledLen = h263->encoder.dummy_buffer_size;
            pBufferHdr->nFlags = OMX_BUFFERFLAG_ENDOFFRAME;
        }
    } else {
        if (h263->encoder.dummy_buffer) {
            TSK_FREE(h263->encoder.dummy_buffer);
        }

        pBufferHdr->nFilledLen = 0;
        h263->decoder.inBufEmptyCount++;
        TSK_DEBUG_INFO("read_frame: no data in queue");
    }
    return OMX_ErrorNone;
}

OMX_ERRORTYPE write_enc_data(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr)
{
    if ((!h263) || (!pBufferHdr || !pBufferHdr->pBuffer)) {
        TSK_DEBUG_ERROR("write_enc_data: Invalid parameter");
        return OMX_ErrorUndefined;
    }

    //TSK_DEBUG_INFO("write_enc_data: pBufferHdr->nFilledLen %d\n", pBufferHdr->nFilledLen);

    if (!h263->encoder.bThreadRunning) {
        TSK_DEBUG_WARN("Encoder thread is not running");
        return OMX_ErrorNone;
    }

    if (pBufferHdr->nFilledLen > 0) {
        h263->encoder.encoded_first_frame = OMX_TRUE;

#ifdef VIDEO_DUMP
    if (h263->video_dump_to_file) {
        if (fpEncodedBufferh263)
            fwrite((void*)pBufferHdr->pBuffer, pBufferHdr->nFilledLen, 1, fpEncodedBufferh263);
    }
#endif
        tdav_codec_h263_encap(h263, (uint8_t*)pBufferHdr->pBuffer, pBufferHdr->nFilledLen);
        h263->encoder.encPacketLoss++;
        pBufferHdr->nFilledLen = 0;
        pBufferHdr->nOffset = 0;

        if(pBufferHdr->nFlags & OMX_BUFFERFLAG_EOS) {
            TSK_DEBUG_INFO("Output EOS.\n");
        }
        return OMX_ErrorNone;
    }

    pBufferHdr->nFilledLen = 0;
    pBufferHdr->nOffset = 0;

    if(pBufferHdr->nFlags & OMX_BUFFERFLAG_EOS) {
        TSK_DEBUG_INFO("Output EOS.\n");
    }
    return OMX_ErrorNone;
}

OMX_ERRORTYPE write_data(tdav_codec_h263_hw_vpu_t *h263, OMX_BUFFERHEADERTYPE *pBufferHdr)
{
    if ((!h263 || !h263->decoder.out_ring_buf) || (!pBufferHdr || !pBufferHdr->pBuffer)) {
        TSK_DEBUG_ERROR("write_data: Invalid parameter");
        return OMX_ErrorUndefined;
    }
    //TSK_DEBUG_INFO("write data: pBufferHdr->nFilledLen %d\n", pBufferHdr->nFilledLen);

    if (!h263->decoder.bThreadRunning) {
        TSK_DEBUG_WARN("Decoder thread is not running");
        return OMX_ErrorNone;
    }

    // Buffer will be invalid, if dynamic port setting is going on.
    if (h263->decoder.bHoldBuffers) {
        TSK_DEBUG_WARN("write_data: Dynamic port setting is going on, buffer will be invalid here");
        pBufferHdr->nFilledLen = 0;
        pBufferHdr->nOffset = 0;
        return OMX_ErrorNone;
    }

    OMX_PTR pBufferOrig = pBufferHdr->pBuffer;
    OMX_U32 size = pBufferHdr->nFilledLen;

    if (size > 0) {
        h263->encoder.decoded_first_frame = OMX_TRUE;

        if ((h263->decoder.context->width != TMEDIA_CODEC_VIDEO(h263)->in.width) ||
            (h263->decoder.context->height != TMEDIA_CODEC_VIDEO(h263)->in.height)) {

            uint8_t *srcPtr = (uint8_t*)pBufferOrig;
            uint8_t *cropPtr = (uint8_t*)h263->decoder.crop_buffer;

            if (cropPtr != NULL) {
                // Process buffer and copy to crop_buffer
                //TODO: Use ProcessBuffer(), if eColorFormat chnages to 420SP
                ProcessBufferForCrop(h263, srcPtr, cropPtr);
                ring_buffer_push(h263->decoder.out_ring_buf, (void*)cropPtr, size);
                pBufferOrig = (OMX_PTR)cropPtr;
            } else {
                ring_buffer_push(h263->decoder.out_ring_buf, (void*)srcPtr, size);
            }
        } else {
            ring_buffer_push(h263->decoder.out_ring_buf, pBufferOrig, size);
        }

#ifdef VIDEO_DUMP
    if (h263->video_dump_to_file) {
        // Use this for short duration calls as this could become huge dump size
        if (fpDecodedBufferh263)
            fwrite(pBufferOrig, size, 1, fpDecodedBufferh263);
    }
#endif

       // TSK_DEBUG_INFO("write_data: Output Q size is [%d]\n", h263->decoder.pOutputMsgQ->Size());
    }
    pBufferHdr->nFilledLen = 0;
    pBufferHdr->nOffset = 0;
    return OMX_ErrorNone;
}

OMX_ERRORTYPE gm_H263EncEventHandler(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_EVENTTYPE eEvent,
        OMX_IN OMX_U32 nData1,
        OMX_IN OMX_U32 nData2,
        OMX_IN OMX_PTR pEventData)
{
    MSG sMsg;
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)pAppData;

    //TSK_DEBUG_ERROR("Client receive event [eEvent:%d, nData1: %d, nData2: %d]\n",
    //        (int)eEvent, (int)nData1, (int)nData2);

    if(eEvent == OMX_EventCmdComplete) {
        h263->encoder.sCmdDone.eEvent = OMX_EventCmdComplete;
        h263->encoder.sCmdDone.nData1 = nData1;
        h263->encoder.sCmdDone.nData2 = nData2;
        h263->encoder.sCmdDone.pEventData = pEventData;
    }
    else if(eEvent == OMX_EventError)
        h263->encoder.bError = OMX_TRUE;
    else {
        sMsg.hComponent = hComponent;
        sMsg.type = EVENT;
        sMsg.data.event.eEvent = eEvent;
        sMsg.data.event.nData1 = nData1;
        sMsg.data.event.nData2 = nData2;
        sMsg.data.event.pEventData = pEventData;
        h263->encoder.pMsgQ->Add(&sMsg);
    }

    return OMX_ErrorNone;
}

OMX_ERRORTYPE gm_H263DecEventHandler(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_EVENTTYPE eEvent,
        OMX_IN OMX_U32 nData1,
        OMX_IN OMX_U32 nData2,
        OMX_IN OMX_PTR pEventData)
{
    MSG sMsg;
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)pAppData;

    //TSK_DEBUG_ERROR("Client receive event [eEvent:%d, nData1: %d, nData2: %d]\n",
    //        (int)eEvent, (int)nData1, (int)nData2);

    if(eEvent == OMX_EventCmdComplete) {
        h263->decoder.sCmdDone.eEvent = OMX_EventCmdComplete;
        h263->decoder.sCmdDone.nData1 = nData1;
        h263->decoder.sCmdDone.nData2 = nData2;
        h263->decoder.sCmdDone.pEventData = pEventData;
    }
    else if(eEvent == OMX_EventError) {
        TSK_DEBUG_ERROR("Client receive event [eEvent:%d, nData1: %d, nData2: %d]",
                        (int)eEvent, (int)nData1, (int)nData2);
        if (!h263->decoder.bError) {
            TSK_DEBUG_WARN("Reseting OMX thread and decoder");
            h263->decoder.bError = OMX_TRUE;
            sMsg.type = EXIT;
            h263->decoder.pMsgQ->Add(&sMsg);
            // Reset Decoder
            start_reset_dec_timer(h263);
        }
    }
    else {
        sMsg.hComponent = hComponent;
        sMsg.type = EVENT;
        sMsg.data.event.eEvent = eEvent;
        sMsg.data.event.nData1 = nData1;
        sMsg.data.event.nData2 = nData2;
        sMsg.data.event.pEventData = pEventData;
        h263->decoder.pMsgQ->Add(&sMsg);
    }

    return OMX_ErrorNone;
}

OMX_ERRORTYPE gm_H263EncEmptyBufferDone(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_BUFFERHEADERTYPE* pBuffer)
{
    MSG sMsg;
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)pAppData;

    //TSK_DEBUG_ERROR("Client receive EncEmptyBufferDone [%p]\n", pBuffer);

    sMsg.hComponent = hComponent;
    sMsg.type = EMPTY_DONE;
    sMsg.data.buffer.pBuffer = pBuffer;
    h263->encoder.pMsgQ->Add(&sMsg);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE gm_H263DecEmptyBufferDone(
        OMX_IN OMX_HANDLETYPE hComponent,
        OMX_IN OMX_PTR pAppData,
        OMX_IN OMX_BUFFERHEADERTYPE* pBuffer)
{
    MSG sMsg;
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)pAppData;

    //TSK_DEBUG_ERROR("Client receive EmptyBufferDone [%p]\n", pBuffer);

    sMsg.hComponent = hComponent;
    sMsg.type = EMPTY_DONE;
    sMsg.data.buffer.pBuffer = pBuffer;
    h263->decoder.pMsgQ->Add(&sMsg);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE gm_H263EncFillBufferDone(
        OMX_OUT OMX_HANDLETYPE hComponent,
        OMX_OUT OMX_PTR pAppData,
        OMX_OUT OMX_BUFFERHEADERTYPE* pBuffer)
{
    MSG sMsg;
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)pAppData;

    //TSK_DEBUG_ERROR("Client receive EncFillBufferDone [%p]\n", pBuffer);

    write_enc_data(h263, pBuffer);

    sMsg.hComponent = hComponent;
    sMsg.type = FILL_DONE;
    sMsg.data.buffer.pBuffer = pBuffer;
    h263->encoder.pMsgQ->Add(&sMsg);
    return OMX_ErrorNone;
}

OMX_ERRORTYPE gm_H263DecFillBufferDone(
        OMX_OUT OMX_HANDLETYPE hComponent,
        OMX_OUT OMX_PTR pAppData,
        OMX_OUT OMX_BUFFERHEADERTYPE* pBuffer)
{
    MSG sMsg;
    tdav_codec_h263_hw_vpu_t *h263 = (tdav_codec_h263_hw_vpu_t*)pAppData;

    //TSK_DEBUG_ERROR("Client receive FillBufferDone [%p]\n", pBuffer);
    if (!h263->decoder.bThreadRunning)
        return OMX_ErrorNone;

    if (pBuffer->nFilledLen)
        write_data(h263, pBuffer);

    sMsg.hComponent = hComponent;
    sMsg.type = FILL_DONE;
    sMsg.data.buffer.pBuffer = pBuffer;
    h263->decoder.pMsgQ->Add(&sMsg);
    return OMX_ErrorNone;
}

OMX_U32 get_enc_component_ports_h263(OMX_HANDLETYPE hComponent)
{
    OMX_PORT_PARAM_TYPE sPortPara;
    OMX_U32 aPorts, vPorts, iPorts, oPorts;

    OMX_INIT_STRUCT(&sPortPara, OMX_PORT_PARAM_TYPE);
    aPorts = vPorts = iPorts = oPorts = 0;

    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamAudioInit, &sPortPara))
        aPorts = sPortPara.nPorts;
    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamVideoInit, &sPortPara))
        vPorts = sPortPara.nPorts;
    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamImageInit, &sPortPara))
        iPorts = sPortPara.nPorts;
    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamOtherInit, &sPortPara))
        oPorts = sPortPara.nPorts;

    return aPorts + vPorts + iPorts + oPorts;
}

OMX_U32 get_component_ports_h263(OMX_HANDLETYPE hComponent)
{
    OMX_PORT_PARAM_TYPE sPortPara;
    OMX_U32 aPorts, vPorts, iPorts, oPorts;

    OMX_INIT_STRUCT(&sPortPara, OMX_PORT_PARAM_TYPE);
    aPorts = vPorts = iPorts = oPorts = 0;

    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamAudioInit, &sPortPara))
        aPorts = sPortPara.nPorts;
    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamVideoInit, &sPortPara))
        vPorts = sPortPara.nPorts;
    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamImageInit, &sPortPara))
        iPorts = sPortPara.nPorts;
    if(OMX_ErrorNone == OMX_GetParameter(hComponent, OMX_IndexParamOtherInit, &sPortPara))
        oPorts = sPortPara.nPorts;

    return aPorts + vPorts + iPorts + oPorts;
}

OMX_ERRORTYPE EncStateTrans(tdav_codec_h263_hw_vpu_t *h263, OMX_STATETYPE eState)
{
    //TSK_DEBUG_INFO("EncStateTrans");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    if ( !h263 || !h263->encoder.hComponent) {
        TSK_DEBUG_ERROR("EncStateTrans: Invalid parameter");
        return ret;
    }

    OMX_COMPONENTTYPE *hComponent = NULL;
    OMX_STATETYPE eCurState = OMX_StateInvalid;

    hComponent = h263->encoder.hComponent;
    OMX_GetState(hComponent, &eCurState);

    if (eCurState == eState) {
        return ret;
    }

    ret = EncSendCommand(h263, OMX_CommandStateSet,eState,NULL, OMX_FALSE);
    if(ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("State trans to %d failed.\n", eState);
        return ret;
    }

    /* Loaded->Idle */
    if(eCurState == OMX_StateLoaded && eState == OMX_StateIdle)
        EncLoad2Idle(h263);
    /* Exec->Idle */
    else if(eCurState == OMX_StateExecuting && eState == OMX_StateIdle)
        h263->encoder.bHoldBuffers = OMX_TRUE;
    /* Pause->Idle */
    else if(eCurState == OMX_StatePause && eState == OMX_StateIdle)
        h263->encoder.bHoldBuffers = OMX_TRUE;
    /* Idle->Loaded */
    else if(eCurState == OMX_StateIdle && eState == OMX_StateLoaded)
        EncIdle2Load(h263);

    TSK_DEBUG_INFO("Called EncWaitCommand in state trans\n");
    ret = EncWaitCommand(h263, OMX_CommandStateSet, eState, NULL);
    if(ret != OMX_ErrorNone)
        return ret;

    TSK_DEBUG_INFO("State trans: [%x] -> [%d] done.\n", eCurState, eState);

    //TSK_DEBUG_INFO("EncStateTrans OUT");
    return ret;
}

OMX_ERRORTYPE StateTrans(tdav_codec_h263_hw_vpu_t *h263, OMX_STATETYPE eState)
{
    //TSK_DEBUG_INFO("StateTrans IN\n");
    if ( !h263 || !h263->decoder.hComponent) {
        TSK_DEBUG_ERROR("StateTrans: Invalid parameter");
        return OMX_ErrorNone;
    }

    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE *hComponent = NULL;
    OMX_STATETYPE eCurState = OMX_StateInvalid;

    hComponent = h263->decoder.hComponent;
    OMX_GetState(hComponent, &eCurState);

    if (eCurState == eState) {
        return ret;
    }

    ret = SendCommand(h263, OMX_CommandStateSet,eState,NULL, OMX_FALSE);
    if(ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("State trans to %d failed.\n", eState);
        return ret;
    }

    /* Loaded->Idle */
    if(eCurState == OMX_StateLoaded && eState == OMX_StateIdle)
        Load2Idle(h263);
    /* Exec->Idle */
    else if(eCurState == OMX_StateExecuting && eState == OMX_StateIdle)
        h263->decoder.bHoldBuffers = OMX_TRUE;
    /* Pause->Idle */
    else if(eCurState == OMX_StatePause && eState == OMX_StateIdle)
        h263->decoder.bHoldBuffers = OMX_TRUE;
    /* Idle->Loaded */
    else if(eCurState == OMX_StateIdle && eState == OMX_StateLoaded)
        Idle2Load(h263);
    else {}

    ret = WaitCommand(h263, OMX_CommandStateSet, eState, NULL);
    if(ret != OMX_ErrorNone)
        return ret;

    //TSK_DEBUG_INFO("State trans: [%x] -> [%d] done.\n", eCurState, eState);

    return ret;
}

OMX_ERRORTYPE EncSendCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData,
        OMX_BOOL bSync)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_HANDLETYPE hComponent = h263->encoder.hComponent;

    h263->encoder.sCmdDone.eEvent = OMX_EventMax;
    h263->encoder.bError = OMX_FALSE;
    ret = OMX_SendCommand(hComponent, Cmd, nParam1, pCmdData);
    if(ret != OMX_ErrorNone)
        return ret;

    if(bSync == OMX_TRUE)
        EncWaitCommand(h263, Cmd, nParam1, pCmdData);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE SendCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData,
        OMX_BOOL bSync)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_HANDLETYPE hComponent = h263->decoder.hComponent;

    h263->decoder.sCmdDone.eEvent = OMX_EventMax;
    h263->decoder.bError = OMX_FALSE;
    ret = OMX_SendCommand(hComponent, Cmd, nParam1, pCmdData);
    if(ret != OMX_ErrorNone)
        return ret;

    if(bSync == OMX_TRUE)
        WaitCommand(h263, Cmd, nParam1, pCmdData);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE EncWaitCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    while(1) {
        if(h263->encoder.sCmdDone.eEvent == OMX_EventCmdComplete
                && h263->encoder.sCmdDone.nData1 == (OMX_U32)Cmd
                && h263->encoder.sCmdDone.nData2 == nParam1
                && h263->encoder.sCmdDone.pEventData == pCmdData)
            break;
        else if(h263->encoder.bError == OMX_TRUE) {
            ret = OMX_ErrorUndefined;
            break;
        }
        else
            usleep(SLEEP_TIME);
    }

    return ret;
}

OMX_ERRORTYPE WaitCommand(
        tdav_codec_h263_hw_vpu_t *h263,
        OMX_COMMANDTYPE Cmd,
        OMX_U32 nParam1,
        OMX_PTR pCmdData)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    while(1) {
        if(h263->decoder.sCmdDone.eEvent == OMX_EventCmdComplete
                && h263->decoder.sCmdDone.nData1 == (OMX_U32)Cmd
                && h263->decoder.sCmdDone.nData2 == nParam1
                && h263->decoder.sCmdDone.pEventData == pCmdData)
            break;
        else if(h263->decoder.bError == OMX_TRUE) {
            ret = OMX_ErrorUndefined;
            break;
        }
        else
            usleep(SLEEP_TIME);
    }

    return ret;
}

OMX_ERRORTYPE EncLoad2Idle(tdav_codec_h263_hw_vpu_t *h263)
{
    OMX_U32 i;
    for(i=0; i<h263->encoder.nPorts; i++)
        prepare_enc_port_buffers(h263, i);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE Load2Idle(tdav_codec_h263_hw_vpu_t *h263)
{
    OMX_U32 i;
    for(i=0; i<h263->decoder.nPorts; i++)
        prepare_port_buffers(h263, i);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE EncIdle2Load(tdav_codec_h263_hw_vpu_t *h263)
{
    OMX_U32 i;

    for(i=0; i<h263->encoder.nPorts; i++)
        free_enc_port_buffers(h263, i);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE Idle2Load(tdav_codec_h263_hw_vpu_t *h263)
{
    OMX_U32 i;

    for(i=0; i<h263->decoder.nPorts; i++)
        free_port_buffers(h263, i);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE unload_enc_component(tdav_codec_h263_hw_vpu_t *h263)
{
    OMX_FreeHandle(h263->encoder.hComponent);
    fsl_osal_thread_destroy(h263->encoder.pThreadId);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE unload_component(tdav_codec_h263_hw_vpu_t *h263)
{
    OMX_FreeHandle(h263->decoder.hComponent);
    fsl_osal_thread_destroy(h263->decoder.pThreadId);

    return OMX_ErrorNone;
}

OMX_ERRORTYPE prepare_enc_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    OMX_COMPONENTTYPE *hComponent = NULL;
    OMX_PARAM_PORTDEFINITIONTYPE sPortDef;
    OMX_BUFFERHEADERTYPE *pBufferHdr = NULL;
    OMX_U8 *pBuffer = NULL;
    OMX_U32 i;

    hComponent = h263->encoder.hComponent;
    OMX_INIT_STRUCT(&sPortDef, OMX_PARAM_PORTDEFINITIONTYPE);
    sPortDef.nPortIndex = nPortIndex;
    OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    //TSK_DEBUG_INFO("Port #%d height: %d, width %d\n", nPortIndex, sPortDef.format.video.nFrameHeight, sPortDef.format.video.nFrameWidth);

    //TSK_DEBUG_INFO("prepare_port_enc_buffers: CFmt 0x%x, SliceHt %d, MIMEType %s, \n",
    //sPortDef.format.video.eColorFormat, sPortDef.format.video.nSliceHeight, sPortDef.format.video.cMIMEType);

    for(i=0; i<sPortDef.nBufferCountActual; i++) {
        if(h263->encoder.bAllocater[nPortIndex] == OMX_TRUE) {
            OMX_AllocateBuffer(hComponent, &pBufferHdr, nPortIndex, NULL, sPortDef.nBufferSize);
            //TSK_DEBUG_INFO("Port #%d Allocate buffer done with buffer size : %d..\n", nPortIndex, sPortDef.nBufferSize);
        }
        else {
            pBuffer = (OMX_U8*)FSL_MALLOC(sPortDef.nBufferSize);
            OMX_UseBuffer(hComponent, &pBufferHdr, nPortIndex, NULL, sPortDef.nBufferSize, pBuffer);
            //TSK_DEBUG_INFO("Port #%d Use buffer done with buffer size : %d.\n", nPortIndex, sPortDef.nBufferSize);
        }
        h263->encoder.pBufferHdr[nPortIndex][i] = pBufferHdr;
    }
    h263->encoder.nBufferHdr[nPortIndex] = sPortDef.nBufferCountActual;

    return OMX_ErrorNone;
}

OMX_ERRORTYPE prepare_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    OMX_COMPONENTTYPE *hComponent = NULL;
    OMX_PARAM_PORTDEFINITIONTYPE sPortDef;
    OMX_BUFFERHEADERTYPE *pBufferHdr = NULL;
    OMX_U8 *pBuffer = NULL;
    OMX_U32 i;

    hComponent = h263->decoder.hComponent;
    OMX_INIT_STRUCT(&sPortDef, OMX_PARAM_PORTDEFINITIONTYPE);
    sPortDef.nPortIndex = nPortIndex;
    OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    TSK_DEBUG_INFO("Dec: Port #%d height: %d, width %d\n",
                    nPortIndex, sPortDef.format.video.nFrameHeight, sPortDef.format.video.nFrameWidth);

    if (sPortDef.format.video.eColorFormat == OMX_COLOR_FormatYUV420SemiPlanar) {
        TSK_DEBUG_INFO("prepare_port_buffers: OMX_COLOR_FormatYUV420SemiPlanar");
        h263->decoder.chromaInterleave = OMX_TRUE;
    }

    for(i=0; i<sPortDef.nBufferCountActual; i++) {
        if(h263->decoder.bAllocater[nPortIndex] == OMX_TRUE) {
            OMX_AllocateBuffer(hComponent, &pBufferHdr, nPortIndex, NULL, sPortDef.nBufferSize);
            //TSK_DEBUG_INFO("Port #%d Allocate buffer done with buffer size : %d..\n", nPortIndex, sPortDef.nBufferSize);
        }
        else {
            pBuffer = (OMX_U8*)FSL_MALLOC(sPortDef.nBufferSize);
            OMX_UseBuffer(hComponent, &pBufferHdr, nPortIndex, NULL, sPortDef.nBufferSize, pBuffer);
            //TSK_DEBUG_INFO("Port #%d Use buffer done with buffer size : %d.\n", nPortIndex, sPortDef.nBufferSize);
        }
        h263->decoder.pBufferHdr[nPortIndex][i] = pBufferHdr;
    }
    h263->decoder.nBufferHdr[nPortIndex] = sPortDef.nBufferCountActual;

    h263->decoder.context->width = sPortDef.format.video.nFrameWidth;
    h263->decoder.context->height = sPortDef.format.video.nFrameHeight;

    if (nPortIndex == kPortIndexOutput) {
        OMX_CONFIG_RECTTYPE rect;
        OMX_INIT_STRUCT(&rect, OMX_CONFIG_RECTTYPE);
        rect.nPortIndex = nPortIndex;

        OMX_GetConfig(hComponent, OMX_IndexConfigCommonOutputCrop, &rect);
        TSK_DEBUG_INFO("prepare_port_buffers: rect: w %d, h %d, (left %d, top %d)",
                        rect.nWidth, rect.nHeight, rect.nLeft, rect.nTop);

        h263->decoder.cropLeft = rect.nLeft;
        h263->decoder.cropTop = rect.nTop;
        h263->decoder.cropRight = rect.nLeft + rect.nWidth;
        h263->decoder.cropBottom = rect.nTop + rect.nHeight;

        // Reset config if rquired
        reset_decoder_config(h263, rect.nWidth, rect.nHeight);
    }
    return OMX_ErrorNone;
}

OMX_ERRORTYPE free_enc_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    //TSK_DEBUG_INFO("free_enc_port_buffers: port #%d free buffer done.\n", nPortIndex);
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE *hComponent;
    OMX_U32 i;

    hComponent = h263->encoder.hComponent;
    for(i=0; i<h263->encoder.nBufferHdr[nPortIndex]; i++) {
        OMX_U8 *ptr = NULL;
        if(h263->encoder.pBufferHdr[nPortIndex][i] == NULL)
            continue;
        ptr = h263->encoder.pBufferHdr[nPortIndex][i]->pBuffer;
        OMX_FreeBuffer(hComponent, nPortIndex, h263->encoder.pBufferHdr[nPortIndex][i]);
        h263->encoder.pBufferHdr[nPortIndex][i] = 0;
        if(h263->encoder.bAllocater[nPortIndex] == OMX_FALSE)
            FSL_FREE(ptr);
    }

    return OMX_ErrorNone;
}

OMX_ERRORTYPE free_port_buffers(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE *hComponent;
    OMX_U32 i;

    hComponent = h263->decoder.hComponent;
    for(i=0; i<h263->decoder.nBufferHdr[nPortIndex]; i++) {
        OMX_U8 *ptr = NULL;
        if(h263->decoder.pBufferHdr[nPortIndex][i] == NULL)
            continue;
        ptr = h263->decoder.pBufferHdr[nPortIndex][i]->pBuffer;
        OMX_FreeBuffer(hComponent, nPortIndex, h263->decoder.pBufferHdr[nPortIndex][i]);
        h263->decoder.pBufferHdr[nPortIndex][i] = 0;
        if(h263->decoder.bAllocater[nPortIndex] == OMX_FALSE)
            FSL_FREE(ptr);
    }

    return OMX_ErrorNone;
}

OMX_ERRORTYPE enc_port_disable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    OMX_COMPONENTTYPE *hComponent;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    hComponent = h263->encoder.hComponent;

    ret = EncSendCommand(h263, OMX_CommandPortDisable, nPortIndex, NULL, OMX_FALSE);
    if(ret != OMX_ErrorNone)
        return ret;

    h263->encoder.bHoldBuffers = OMX_TRUE;

    free_enc_port_buffers(h263, nPortIndex);

    EncWaitCommand(h263, OMX_CommandPortDisable, nPortIndex, NULL);

    TSK_DEBUG_ERROR("port disbale done.\n");

    return OMX_ErrorNone;
}

OMX_ERRORTYPE port_disable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    OMX_COMPONENTTYPE *hComponent;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    hComponent = h263->decoder.hComponent;

    ret = SendCommand(h263, OMX_CommandPortDisable, nPortIndex, NULL, OMX_FALSE);
    if(ret != OMX_ErrorNone)
        return ret;

    h263->decoder.bHoldBuffers = OMX_TRUE;

    free_port_buffers(h263, nPortIndex);

    WaitCommand(h263, OMX_CommandPortDisable, nPortIndex, NULL);

    //TSK_DEBUG_INFO("port disbale done.\n");

    return OMX_ErrorNone;
}

OMX_ERRORTYPE enc_port_enable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    TSK_DEBUG_ERROR("enc_port_enable \n");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE *hComponent;
    OMX_U32 i;

    hComponent = h263->encoder.hComponent;
    ret = EncSendCommand(h263, OMX_CommandPortEnable, nPortIndex, NULL, OMX_FALSE);
    if(ret != OMX_ErrorNone)
        return ret;

    prepare_enc_port_buffers(h263, nPortIndex);

    EncWaitCommand(h263, OMX_CommandPortEnable, nPortIndex, NULL);

    h263->encoder.bHoldBuffers = OMX_FALSE;
    for(i=0; i<h263->encoder.nBufferHdr[nPortIndex]; i++) {
        if(h263->encoder.PortDir[nPortIndex] == OMX_DirInput) {
            TSK_DEBUG_ERROR("enc_port_enable: OMX_DirInput");
            read_enc_frame(h263, h263->encoder.pBufferHdr[nPortIndex][i]);
            OMX_EmptyThisBuffer(h263->encoder.hComponent, h263->encoder.pBufferHdr[nPortIndex][i]);
        }
    }

    TSK_DEBUG_ERROR("enc port enable done.\n");

    return OMX_ErrorNone;
}

OMX_ERRORTYPE port_enable(tdav_codec_h263_hw_vpu_t *h263, OMX_U32 nPortIndex)
{
    //TSK_DEBUG_INFO("port_enable \n");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE *hComponent;
    OMX_U32 i;

    hComponent = h263->decoder.hComponent;
    ret = SendCommand(h263, OMX_CommandPortEnable, nPortIndex, NULL, OMX_FALSE);
    if(ret != OMX_ErrorNone)
        return ret;

    prepare_port_buffers(h263, nPortIndex);

    WaitCommand(h263, OMX_CommandPortEnable, nPortIndex, NULL);

    h263->decoder.bHoldBuffers = OMX_FALSE;
    for(i=0; i<h263->decoder.nBufferHdr[nPortIndex]; i++) {
        if(h263->decoder.PortDir[nPortIndex] == OMX_DirInput) {
            //TSK_DEBUG_INFO("port_enable: OMX_DirInput");
            read_frame(h263, h263->decoder.pBufferHdr[nPortIndex][i]);
            OMX_EmptyThisBuffer(h263->decoder.hComponent, h263->decoder.pBufferHdr[nPortIndex][i]);
        }
    }

    return OMX_ErrorNone;
}

static int close_hw_vpu_encoder(tdav_codec_h263_hw_vpu_t *h263)
{
    TSK_DEBUG_INFO("close_hw_vpu_encoder IN \n");
    // Send exit to enc_omx_process_thread
    MSG sMsg;
    sMsg.type = EXIT;
    h263->encoder.pMsgQ->Add(&sMsg);

#if 0
    if (h263->encoder.prev_frame) {
        FSL_FREE(h263->encoder.prev_frame);
        h263->encoder.prev_frame = NULL;
    }
#endif

    if (h263->encoder.dummy_decode_buffer) {
        TSK_FREE(h263->encoder.dummy_decode_buffer);
    }
    if (h263->encoder.dummy_buffer) {
        TSK_FREE(h263->encoder.dummy_buffer);
    }

    EncStateTrans(h263, OMX_StateIdle);
    EncStateTrans(h263, OMX_StateLoaded);
    unload_enc_component(h263);
    TSK_DEBUG_INFO("close_hw_vpu_encoder: Enc pkt loss %d\n", h263->encoder.encPacketLoss);
    h263->encoder.pMsgQ->Free();
    ring_buffer_free(h263->encoder.in_ring_buf);
    h263->encoder.bClosedEncoder = OMX_TRUE;
    TSK_DEBUG_INFO("close_hw_vpu_encoder OUT \n");
    return 0;
}

static int close_hw_vpu_decoder(tdav_codec_h263_hw_vpu_t *h263)
{
    TSK_DEBUG_INFO("close_hw_vpu_decoder IN \n");
    // Send exit to omx_process_thread
    MSG sMsg;
    sMsg.type = EXIT;
    h263->decoder.pMsgQ->Add(&sMsg);

    if (h263->decoder.prev_frame) {
        FSL_FREE(h263->decoder.prev_frame);
        h263->decoder.prev_frame = NULL;
    }
    StateTrans(h263, OMX_StateIdle);
    StateTrans(h263, OMX_StateLoaded);
    unload_component(h263);
    h263->decoder.pMsgQ->Free();

    TSK_DEBUG_INFO("close_hw_vpu_deccoder: Dec pkt loss %d\n", h263->decoder.decPacketLoss);

    ring_buffer_free(h263->decoder.in_ring_buf);
    ring_buffer_free(h263->decoder.out_ring_buf);
    h263->decoder.bClosedDecoder = OMX_TRUE;
    TSK_DEBUG_INFO("close_hw_vpu_decoder OUT \n");
    return 0;
}

static int hw_vpu_open_encoder(tdav_codec_h263_hw_vpu_t* self)
{
    if (!self) {
        TSK_DEBUG_ERROR("hw_vpu_open_encoder: Invalid parameter");
        return -1;
    }
    tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;

    //TSK_DEBUG_INFO("hw_vpu_open_encoder IN \n");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_HANDLETYPE hComponent = NULL;
    OMX_PARAM_PORTDEFINITIONTYPE sPortDef;
    OMX_U32 i=0;
    efsl_osal_return_type_t fsl_ret;
    QUEUE_ERRORTYPE queue_ret = QUEUE_SUCCESS;
    OMX_S32 gm_video_bitRate = 0;
    OMX_S32 req_frame_rate =0;
    int factor=0, fps=0;
    int max_buffer_size = 0;
    int max_capacity = 0;
    char propValue[PROPERTY_VALUE_MAX];
    int ring_buff_cap = 0;

    // Create input and output queues
    h263->encoder.pMsgQ = FSL_NEW(Queue, ());
    if(h263->encoder.pMsgQ == NULL) {
        TSK_DEBUG_ERROR("Error creating event Queue instance.\n");
        goto free;
    }
    queue_ret = h263->encoder.pMsgQ->Create((fsl_osal_u32)MAX_QUEUE_SIZE, (fsl_osal_u32)sizeof(MSG), (efsl_osal_bool)(E_FSL_OSAL_TRUE));
    if (queue_ret != QUEUE_SUCCESS) {
        TSK_DEBUG_ERROR("Create event message queue failed.\n");
        goto free;
    }

    //create ring buffer
    // Ring buffer capacity
    h263->encoder.in_ring_buf_cap = MAX_IN_ENC_RB_CAP;
    if (property_get("ringbuf_enc_in_size", propValue, NULL)) {
        ring_buff_cap = atoi(propValue);
        if (ring_buff_cap > 0) {
            h263->encoder.in_ring_buf_cap = ring_buff_cap;
        }
    }

    // Ring buffer for input raw data from camera
    // Create ring buffer with max decoded buffer size
    max_buffer_size = (h263->encoder.context->width * h263->encoder.context->height *3) >> 1;
    max_capacity = h263->encoder.in_ring_buf_cap;
    h263->encoder.in_ring_buf = ring_buffer_init(max_capacity, max_buffer_size);

    if (h263->encoder.in_ring_buf == NULL) {
        TSK_DEBUG_ERROR("Error allocating input ring buffer for encoder");
        goto free;
    }

    h263->encoder.width = h263->encoder.context->width;
    h263->encoder.height = h263->encoder.context->height;
    factor = h263->encoder.bitrate_fact;
    fps = TMEDIA_CODEC_VIDEO(self)->out.fps;
    // Set bit rate to auto (0) by setting bitrate factor to 0 in config file
    h263->encoder.bitrate = (h263->encoder.width * h263->encoder.height * fps * factor * 0.07);

    TSK_DEBUG_WARN("hw_vpu_open_encoder: h263->encoder.bitrate %d \n", h263->encoder.bitrate);

    //Initialize
    h263->encoder.name = (OMX_STRING) VPU_ENC_COMP_NAME_AVCENC;
    h263->encoder.role = (OMX_STRING) VPU_ENC_COMP_ROLE_AVCENC;

    ret = OMX_GetHandle(&hComponent, h263->encoder.name, h263, &gH263EncCallBacks);
    if(ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Load component %s failed.\n", h263->encoder.name);
        goto free;
    }

    OMX_PARAM_COMPONENTROLETYPE CurRole;
    OMX_INIT_STRUCT(&CurRole, OMX_PARAM_COMPONENTROLETYPE);
    fsl_ret = fsl_osal_memcpy(&CurRole.cRole, h263->encoder.role, OMX_MAX_STRINGNAME_SIZE);
    if (fsl_ret != E_FSL_OSAL_SUCCESS) {
        TSK_DEBUG_ERROR("Unable to do memcpy, invalid src or dst");
        goto free;
    }

    ret = OMX_SetParameter(hComponent, OMX_IndexParamStandardComponentRole, &CurRole);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to set components role");
        goto free;
    }

    OMX_INIT_STRUCT(&sPortDef, OMX_PARAM_PORTDEFINITIONTYPE);
    sPortDef.nPortIndex = kPortIndexInput;
    ret = OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to get components port def");
        goto free;
    }

    sPortDef.format.video.nFrameWidth = h263->encoder.width;
    sPortDef.format.video.nFrameHeight = h263->encoder.height;
    sPortDef.format.video.xFramerate = TMEDIA_CODEC_VIDEO(self)->out.fps * Q16_SHIFT;
    sPortDef.nBufferSize = h263->encoder.width * h263->encoder.height * 3 / 2;
    ret = OMX_SetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to set components port def");
        goto free;
    }

    // Check width, height, frame rate and buffer size
    ret = OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("2 Unable to get components port def");
        goto free;
    }

    req_frame_rate = TMEDIA_CODEC_VIDEO(self)->out.fps;

    TSK_DEBUG_INFO("encoder: W %d, H %d, FR %d, BufSize %d",
                    sPortDef.format.video.nFrameWidth,
                    sPortDef.format.video.nFrameHeight,
                    sPortDef.format.video.xFramerate / Q16_SHIFT, sPortDef.nBufferSize);

    OMX_VIDEO_PARAM_BITRATETYPE sBitrate;
    gm_video_bitRate = h263->encoder.bitrate;

    OMX_INIT_STRUCT(&sBitrate, OMX_VIDEO_PARAM_BITRATETYPE);
    sBitrate.nPortIndex = kPortIndexOutput;
    ret = OMX_GetParameter(hComponent, OMX_IndexParamVideoBitrate, &sBitrate);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to get components bit rate");
        goto free;
    }

    sBitrate.eControlRate = OMX_Video_ControlRateVariable; //OMX_Video_ControlRateConstant;
    sBitrate.nTargetBitrate = gm_video_bitRate;
    ret = OMX_SetParameter(hComponent, OMX_IndexParamVideoBitrate, &sBitrate);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to set components bit rate");
        goto free;
    }

    TSK_DEBUG_INFO("encoder: Bitrate %d", sBitrate.nTargetBitrate);

    OMX_INIT_STRUCT(&sPortDef, OMX_PARAM_PORTDEFINITIONTYPE);
    sPortDef.nPortIndex = kPortIndexOutput;
    ret = OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to get components port def for %d", kPortIndexOutput);
        goto free;
    }

    sPortDef.format.video.nBitrate = gm_video_bitRate;
    ret = OMX_SetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to set components port def for %d", kPortIndexOutput);
        goto free;
    }

    h263->encoder.hComponent = (OMX_COMPONENTTYPE*)hComponent;
    h263->encoder.nPorts = get_enc_component_ports_h263(hComponent);
    OMX_INIT_STRUCT(&sPortDef, OMX_PARAM_PORTDEFINITIONTYPE);
    for(i=0; i<h263->encoder.nPorts; i++) {
        sPortDef.nPortIndex = i;
        ret = OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
        if (ret != OMX_ErrorNone) {
            TSK_DEBUG_ERROR("Unable to get components port def for %d", i);
            goto free;
        }

        h263->encoder.PortDir[i] = sPortDef.eDir;
        if(h263->encoder.PortDir[i] == OMX_DirInput)
            h263->encoder.bAllocater[i] = OMX_TRUE;  //in buffer allocated by vpu component
        if(h263->encoder.PortDir[i] == OMX_DirOutput)
            h263->encoder.bAllocater[i] = OMX_TRUE;   // out buffer allocated by component
    }

    //Set GOP size
    OMX_VIDEO_PARAM_H263TYPE h263type;
    OMX_INIT_STRUCT(&h263type, OMX_VIDEO_PARAM_H263TYPE);
    h263type.nPortIndex = kPortIndexOutput;

    ret = OMX_GetParameter(hComponent, OMX_IndexParamVideoH263, &h263type);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("p-Frame: Unable to get components H263 param for port %d", kPortIndexOutput);
    }
    TSK_DEBUG_INFO("encoder: GetParam: P-Frame %d, B-Frame %d", h263type.nPFrames, h263type.nBFrames);

    h263type.nAllowedPictureTypes =
        OMX_VIDEO_PictureTypeI | OMX_VIDEO_PictureTypeP;

    h263type.nPFrames = h263->encoder.gop_size;
    if (h263type.nPFrames == 0) {
        h263type.nAllowedPictureTypes = OMX_VIDEO_PictureTypeI;
    }
    h263type.nBFrames = 0;
    h263type.bPLUSPTYPEAllowed = OMX_FALSE;
    h263type.bForceRoundingTypeToZero = OMX_FALSE;
    h263type.nPictureHeaderRepetition = 0;
    h263type.nGOBHeaderInterval = 0;

    ret = OMX_SetParameter(hComponent, OMX_IndexParamVideoH263, &h263type);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("P-Frame: Unable to set components AVC param");
    }
    // End of GOP

    TSK_DEBUG_WARN("'%s' loaded with (%dx%d) resolution and pFrame %d",
                    h263->encoder.name,
                    sPortDef.format.video.nFrameWidth,
                    sPortDef.format.video.nFrameHeight,
                    h263type.nPFrames+1);

    ret = EncStateTrans(h263, OMX_StateIdle);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to do state trans to Idle");
        goto free;
    }
    ret = EncStateTrans(h263, OMX_StateExecuting);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to do state trans to Exec");
        goto free;
    }

    h263->encoder.bClosedEncoder = OMX_FALSE;
    fsl_ret = fsl_osal_thread_create(&h263->encoder.pThreadId, NULL, enc_omx_process_thread, h263);
    if (fsl_ret != E_FSL_OSAL_SUCCESS) {
        TSK_DEBUG_ERROR("Unable to create encoder thread");
        goto free;
    }

    h263->encoder.bInitailBufferSubmit = OMX_TRUE;
    h263->encoder.prev_frame = NULL;
    h263->encoder.source_done = OMX_FALSE;
    // TODO: Update maxBufInQueue, when requested by Phone app
    h263->encoder.maxBufInQueue = MAX_ENC_BUF_TO_KEEP;
    TSK_DEBUG_INFO("hw_vpu_open_encoder OUT");
    return (int)ret;

free:
    free(h263);
    ring_buffer_free(h263->encoder.in_ring_buf);
    OMX_FreeHandle(h263->encoder.hComponent);
    h263->encoder.hComponent = NULL;
    return (int)ret;
}

static int hw_vpu_open_decoder(tdav_codec_h263_hw_vpu_t* self)
{
    if (!self) {
        TSK_DEBUG_ERROR("hw_vpu_open_decoder: Invalid parameter");
        return -1;
    }
    tdav_codec_h263_hw_vpu_t* h263 = (tdav_codec_h263_hw_vpu_t*)self;

    TSK_DEBUG_INFO("hw_vpu_open_decoder IN");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    QUEUE_ERRORTYPE queue_ret = QUEUE_SUCCESS;
    int max_buffer_size = 0;
    int max_capacity = 0;
    char propValue[PROPERTY_VALUE_MAX];
    int ring_buff_cap = 0;

    // Create input and output queues
    h263->decoder.pMsgQ = FSL_NEW(Queue, ());
    if(h263->decoder.pMsgQ == NULL) {
        TSK_DEBUG_ERROR("Error creating event Queue instance");
        goto free;
    }
    queue_ret = h263->decoder.pMsgQ->Create((fsl_osal_u32)MAX_QUEUE_SIZE, (fsl_osal_u32)sizeof(MSG), (efsl_osal_bool)(E_FSL_OSAL_TRUE));
    if (queue_ret != QUEUE_SUCCESS) {
        TSK_DEBUG_ERROR("Create event message queue failed.\n");
        goto free;
    }

    //create ring buffer
    // Ring buffer capacity
    h263->decoder.in_ring_buf_cap = MAX_IN_DEC_RB_CAP;
    if (property_get("ringbuf_dec_in_size", propValue, NULL)) {
        ring_buff_cap = atoi(propValue);
        if (ring_buff_cap > 0) {
            h263->decoder.in_ring_buf_cap = ring_buff_cap;
        }
    }

    // Ring buffer for input RTP data (encoded data)
    // Create ring buffer with max encoded buffer size
    max_buffer_size = (h263->decoder.context->width * h263->decoder.context->height *3) >> 3;
    max_capacity = h263->decoder.in_ring_buf_cap;

    h263->decoder.in_ring_buf = ring_buffer_init(max_capacity, max_buffer_size);
    if (h263->decoder.in_ring_buf == NULL) {
        TSK_DEBUG_ERROR("Error allocating input ring buffer for decoder");
        goto free;
    }

    // Ring buffer for output decoded data
    // Ring buffer capacity
    h263->decoder.out_ring_buf_cap = MAX_OUT_DEC_RB_CAP;
    if (property_get("ringbuf_dec_out_size", propValue, NULL)) {
        ring_buff_cap = atoi(propValue);
        if (ring_buff_cap > 0) {
            h263->decoder.out_ring_buf_cap = ring_buff_cap;
        }
    }

    // Create ring buffer with max decoded buffer size
    max_buffer_size = (h263->decoder.context->width * h263->decoder.context->height *3) >> 1;
    max_capacity = h263->decoder.out_ring_buf_cap;

    h263->decoder.out_ring_buf = ring_buffer_init(max_capacity, max_buffer_size);
    if (h263->decoder.out_ring_buf == NULL) {
        TSK_DEBUG_ERROR("Error allocating output ring buffer for decoder");
        goto free;
    }

    ret = open_fsl_omx_decoder(h263);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Error opening omx h263 codec");
        goto free;
    }

    TSK_DEBUG_INFO("hw_vpu_open_decoder OUT");
    return (int)ret;

free:
    // Free queue
    free(h263);
    ring_buffer_free(h263->decoder.in_ring_buf);
    ring_buffer_free(h263->decoder.out_ring_buf);
    return (int)ret;
}

static OMX_ERRORTYPE open_fsl_omx_decoder(tdav_codec_h263_hw_vpu_t* h263)
{
    TSK_DEBUG_INFO("open_fsl_omx_decoder");
    if (!h263) {
        TSK_DEBUG_ERROR("open_fsl_omx_decoder: Invalid parameter passed");
        return OMX_ErrorUndefined;
    }

    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_HANDLETYPE hComponent = NULL;
    OMX_PARAM_PORTDEFINITIONTYPE sPortDef;
    OMX_U32 i;
    QUEUE_ERRORTYPE queue_ret = QUEUE_SUCCESS;
    efsl_osal_return_type_t fsl_ret;

    //Initialize
    h263->decoder.name = (OMX_STRING) VPU_DEC_COMP_NAME_AVCDEC;
    h263->decoder.role = (OMX_STRING) VPU_DEC_COMP_ROLE_AVCDEC;

    ret = OMX_GetHandle(&hComponent, h263->decoder.name, h263, &gH263DecCallBacks);
    if(ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Load component %s failed.\n", h263->decoder.name);
        goto free;
    }

    OMX_PARAM_COMPONENTROLETYPE CurRole;
    OMX_INIT_STRUCT(&CurRole, OMX_PARAM_COMPONENTROLETYPE);
    fsl_osal_memcpy(&CurRole.cRole, h263->decoder.role, OMX_MAX_STRINGNAME_SIZE);
    ret = OMX_SetParameter(hComponent, OMX_IndexParamStandardComponentRole, &CurRole);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to set component role");
        goto free;
    }

    h263->decoder.hComponent = (OMX_COMPONENTTYPE*)hComponent;
    h263->decoder.nPorts = get_component_ports_h263(hComponent);
    OMX_INIT_STRUCT(&sPortDef, OMX_PARAM_PORTDEFINITIONTYPE);
    for(i=0; i<h263->decoder.nPorts; i++) {
        sPortDef.nPortIndex = i;
        ret = OMX_GetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
        if (ret != OMX_ErrorNone) {
            TSK_DEBUG_ERROR("Unable to get components port def");
            goto free;
        }

        TSK_DEBUG_INFO("open_fsl_omx_decoder: CFmt %x", sPortDef.format.video.eColorFormat);

        h263->decoder.PortDir[i] = sPortDef.eDir;
        if(h263->decoder.PortDir[i] == OMX_DirInput) {
            h263->decoder.bAllocater[i] = OMX_FALSE;  //in buffer allocated by client
            sPortDef.format.video.eCompressionFormat = OMX_VIDEO_CodingH263;
            sPortDef.format.video.nFrameWidth = h263->decoder.context->width;
            sPortDef.format.video.nFrameHeight = h263->decoder.context->height;

            ret = OMX_SetParameter(hComponent, OMX_IndexParamPortDefinition, &sPortDef);
            if (ret != OMX_ErrorNone) {
                TSK_DEBUG_ERROR("Unable to set components port def");
                goto free;
            }

	    TSK_DEBUG_INFO("open_fsl_omx_decoder: set width to %d, and hight to %d\n",
	     h263->decoder.context->width, h263->decoder.context->height);
        }
        if(h263->decoder.PortDir[i] == OMX_DirOutput)
            h263->decoder.bAllocater[i] = OMX_TRUE;   // out buffer allocated by component
    }

    TSK_DEBUG_WARN("open_fsl_omx_decoder: Load component %s done with resolution (%dx%d)",
                    h263->decoder.name,
                    sPortDef.format.video.nFrameWidth,
                    sPortDef.format.video.nFrameHeight);

    ret = StateTrans(h263, OMX_StateIdle);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to do state trans to Idle");
        goto free;
    }
    ret = StateTrans(h263, OMX_StateExecuting);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("Unable to do state trans to Exec");
        goto free;
    }

    h263->decoder.bClosedDecoder = OMX_FALSE;
    fsl_ret = fsl_osal_thread_create(&h263->decoder.pThreadId, NULL, omx_process_thread, h263);
    if (fsl_ret != E_FSL_OSAL_SUCCESS) {
        TSK_DEBUG_ERROR("Unable to create decoder thread");
        goto free;
    }

    h263->decoder.bInitailBufferSubmit = OMX_TRUE;
    h263->decoder.prev_frame = NULL;
    h263->decoder.source_done = OMX_FALSE;
    // TODO: Update maxBufInQueue, when requested by Phone app
    h263->decoder.maxBufInQueue = MAX_DEC_BUF_TO_KEEP;
    h263->decoder.chromaInterleave = OMX_FALSE;
    h263->decoder.decoder_reset = tsk_false;
    TSK_DEBUG_INFO("open_fsl_omx_decoder OUT");
    return ret;

free:
    OMX_FreeHandle(h263->decoder.hComponent);
    h263->decoder.hComponent = NULL;
    return ret;
}

/* Start encode process, if not started.
 * Force push buffer to OMX encoder if not passed during read request.
 * Push raw data to ring buffer and return.
 * Encoded data will be made availble in write_enc_data().
 * @return, return -1 if error, otherwise 0.
*/
static int hw_vpu_encode(tdav_codec_h263_hw_vpu_t* self, const AVPacket *avpkt)
{
    if (!self || !avpkt) {
        TSK_DEBUG_ERROR("hw_vpu_encode: Invalid parameter");
        return -1;
    }

    // copy to ensure we do not change avpkt
    AVPacket pkt = *avpkt;
    if (!self->encoder.source_done) {
        if (!self->encoder.in_ring_buf || !avpkt->data) {
            TSK_DEBUG_WARN("hw_vpu_encode: Invalid data/rb for encode");
            return -1;
        }
        // Fed buffer to encoder, if missed in read_enc_frame()
        if (self->encoder.inBufEmptyCount >= 1) {
            TSK_DEBUG_INFO("hw_vpu_encode:Ring buf count %d", self->encoder.in_ring_buf->count);
            if (self->encoder.in_ring_buf->count >= ENC_MAX_PORT_BUF) {
                for(int i=0; i<self->encoder.nBufferHdr[0]; i++) {
                    read_enc_frame(self, self->encoder.pBufferHdr[0][i]);
                    OMX_EmptyThisBuffer(self->encoder.hComponent, self->encoder.pBufferHdr[0][i]);
                 }
            } else if (self->encoder.in_ring_buf->count >= ENC_MIN_PORT_BUF) {
                read_enc_frame(self, self->encoder.pBufferHdr[0][0]);
                OMX_EmptyThisBuffer(self->encoder.hComponent, self->encoder.pBufferHdr[0][0]);
             }
        }

        // Push incoming data from camera source to ring buffer
        if (ring_buffer_push(self->encoder.in_ring_buf, avpkt->data, avpkt->size)) {
            // Increment enc packet
            self->encoder.encPacketLoss++;
         }
     }

    if (self->encoder.bInitailBufferSubmit) {
        OMX_STATETYPE eState = OMX_StateInvalid;
        OMX_GetState(self->encoder.hComponent, &eState);

        if (eState == OMX_StateExecuting) {
            //TSK_DEBUG_INFO("hw_vpu_encode: initial buffer submit\n");
            enc_start_data_process(self);
            self->encoder.bInitailBufferSubmit = OMX_FALSE;
        }
    }

    return 0;
}

/* Start decode process, if not started.
 * Push RTP encoded data to ring buffer and return.
 * Decoded data will be read in copyDecodedData() if availble.
 * @return, return -1 if error, otherwise 0.
*/
static int hw_vpu_decode(tdav_codec_h263_hw_vpu_t* self, const AVPacket *avpkt)
{
    if (!self || !avpkt) {
        TSK_DEBUG_ERROR("hw_vpu_decode: Invalid parameter");
        return -1;
    }

    // copy to ensure we do not change avpkt
    AVPacket pkt = *avpkt;
    int queue_size;

    // Increment dec packet
    self->decoder.decPacketLoss++;

    if (!self->decoder.source_done) {
        if (!self->decoder.in_ring_buf || !avpkt->data) {
            TSK_DEBUG_WARN("hw_vpu_decode: Invalid data/rb for decode");
             return -1;
         }

        if (ring_buffer_push(self->decoder.in_ring_buf, avpkt->data, avpkt->size)) {
            // Increment dec packet
            self->decoder.decPacketLoss++;
         }

        // Start decoder process
        if (self->decoder.bInitailBufferSubmit) {
            OMX_STATETYPE eState = OMX_StateInvalid;
            OMX_GetState(self->decoder.hComponent, &eState);
            if (eState == OMX_StateExecuting) {
                TSK_DEBUG_INFO("hw_vpu_decode: initial buffer submit\n");
                start_data_process(self);
                self->decoder.bInitailBufferSubmit = OMX_FALSE;
            }
         }
     }

    return 0;
}

/*
 * Decoded data will be read in copyDecodedData() if availble.
 * @return, return -1 if error, otherwise 0.
*/
static void copyDecodedData(tdav_codec_h263_hw_vpu_t* self, void** out_data, int size)
{
    if (!(*out_data) || !(self)) {
        TSK_DEBUG_WARN("copyDecodedData: Invalid *out_data or self ptr");
        return;
    }

    //TODO: Check if self->decoder.prev_frame is required
    bool realloc = tsk_true; // re-alloc out_data if required

    ring_buffer_pop(self->decoder.out_ring_buf, *out_data, &size, realloc);

    if (size > 0) {
        // Decrement dec packet
        self->decoder.decPacketLoss--;
     }
}

static void * enc_omx_process_thread(void *ptr)
{
    TSK_DEBUG_INFO("\n enc_omx_process_thread IN \n");
    tdav_codec_h263_hw_vpu_t *self = (tdav_codec_h263_hw_vpu_t*)ptr;
    OMX_BUFFERHEADERTYPE *pBufferHdr = NULL;
    MSG sMsg;

    while(!self->encoder.bClosedEncoder) {
        self->encoder.bThreadRunning = OMX_TRUE;
        self->encoder.pMsgQ->Get(&sMsg);
        if(sMsg.type == EVENT) {
            enc_process_event(self, &sMsg);
        }
        else if(sMsg.type == EMPTY_DONE) {
            pBufferHdr = sMsg.data.buffer.pBuffer;

            if(self->encoder.bHoldBuffers != OMX_TRUE && self->encoder.bInEos != OMX_TRUE) {
               read_enc_frame(self, pBufferHdr);
               if(pBufferHdr->nFilledLen)
                   OMX_EmptyThisBuffer(sMsg.hComponent, pBufferHdr);
            }
        }
        else if(sMsg.type == FILL_DONE) {
            pBufferHdr = sMsg.data.buffer.pBuffer;
            if(self->encoder.bHoldBuffers != OMX_TRUE) {
                OMX_FillThisBuffer(sMsg.hComponent, pBufferHdr);
            }
        }
        else if(sMsg.type == EXIT) {
            TSK_DEBUG_INFO("sMsg.type == EXIT \n");
            break;
        }
    }
    self->encoder.bThreadRunning = OMX_FALSE;
    self->encoder.source_done = OMX_TRUE;

    TSK_DEBUG_INFO("enc_omx_process_thread OUT \n");
    return NULL;
}

static void * omx_process_thread(void *ptr)
{
    TSK_DEBUG_INFO("omx_process_thread IN \n");
    tdav_codec_h263_hw_vpu_t *self = (tdav_codec_h263_hw_vpu_t*)ptr;
    OMX_BUFFERHEADERTYPE *pBufferHdr = NULL;
    MSG sMsg;

    while(!self->decoder.bClosedDecoder) {
        self->decoder.bThreadRunning = OMX_TRUE;
        self->decoder.pMsgQ->Get(&sMsg);
        if(sMsg.type == EVENT) {
            process_event(self, &sMsg);
            while(self->decoder.in_ring_buf->count < 0) {
                // sleep for 10ms to get input data
                //TSK_DEBUG_INFO("in EVENT waiting for input Q to fill\n");
                usleep(SLEEP_TIME);
            }
        }
        else if(sMsg.type == EMPTY_DONE) {
            pBufferHdr = sMsg.data.buffer.pBuffer;

            if(self->decoder.bHoldBuffers != OMX_TRUE && self->decoder.bInEos != OMX_TRUE) {
               read_frame(self, pBufferHdr);
               OMX_EmptyThisBuffer(sMsg.hComponent, pBufferHdr);
            }
        }
        else if(sMsg.type == FILL_DONE) {
            pBufferHdr = sMsg.data.buffer.pBuffer;
	    if(self->decoder.bHoldBuffers != OMX_TRUE) {
                //TSK_DEBUG_INFO("omx_process_thread: calling fillbuffer \n");
                OMX_FillThisBuffer(sMsg.hComponent, pBufferHdr);
            }
        }
        else if(sMsg.type == EXIT) {
            TSK_DEBUG_INFO("sMsg.type == EXIT \n");
            break;
        }
    }
    self->decoder.bThreadRunning = OMX_FALSE;
    self->decoder.source_done = OMX_TRUE;

    TSK_DEBUG_INFO("omx_process_thread OUT \n");
    return NULL;
}

OMX_ERRORTYPE enc_start_data_process(tdav_codec_h263_hw_vpu_t *h263)
{
    //TSK_DEBUG_INFO("enc_start_data_process");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_U32 i;

    h263->encoder.bHoldBuffers = OMX_FALSE;

    /* Send output buffers */
    for(i=0; i<h263->encoder.nBufferHdr[1]; i++) {
        h263->encoder.pBufferHdr[1][i]->nFilledLen = 0;
        h263->encoder.pBufferHdr[1][i]->nOffset = 0;
        OMX_FillThisBuffer(h263->encoder.hComponent, h263->encoder.pBufferHdr[1][i]);
    }

    if (h263->encoder.bInitailBufferSubmit == true) {
        read_enc_frame(h263, h263->encoder.pBufferHdr[0][0]);
        h263->encoder.dummy_frame = h263->encoder.pBufferHdr[0][0];
        for(i=0; i < 3; i++) {
            TSK_DEBUG_INFO("Enc - OMX_EmptyThisBuffer i %d", (i+1));
            OMX_EmptyThisBuffer(h263->encoder.hComponent, h263->encoder.dummy_frame);
        }
    } else {
        /* Send input buffers */
        for(i=0; i<h263->encoder.nBufferHdr[0]; i++) {
            read_enc_frame(h263, h263->encoder.pBufferHdr[0][i]);
            OMX_EmptyThisBuffer(h263->encoder.hComponent, h263->encoder.pBufferHdr[0][i]);
        }
    }
    return ret;
}

OMX_ERRORTYPE start_data_process(tdav_codec_h263_hw_vpu_t *h263)
{
    //TSK_DEBUG_INFO("start_data_process");
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_U32 i;

    h263->decoder.bHoldBuffers = OMX_FALSE;

    /* Send output buffers */
    for(i=0; i<h263->decoder.nBufferHdr[1]; i++) {
        h263->decoder.pBufferHdr[1][i]->nFilledLen = 0;
        h263->decoder.pBufferHdr[1][i]->nOffset = 0;
        OMX_FillThisBuffer(h263->decoder.hComponent, h263->decoder.pBufferHdr[1][i]);
    }

    /* Send input buffers */
    for(i=0; i<h263->decoder.nBufferHdr[0]; i++) {
        read_frame(h263, h263->decoder.pBufferHdr[0][i]);
        OMX_EmptyThisBuffer(h263->decoder.hComponent, h263->decoder.pBufferHdr[0][i]);
    }
    return ret;
}

OMX_ERRORTYPE process_event(tdav_codec_h263_hw_vpu_t *self, MSG *pMsg)
{
    //TSK_DEBUG_INFO("\nprocess_event IN\n");
    OMX_EVENTTYPE event = pMsg->data.event.eEvent;
    OMX_U32 nPortIndex = pMsg->data.event.nData1;

    if(event == OMX_EventPortSettingsChanged) {
        port_disable(self, nPortIndex);
        port_enable(self, nPortIndex);

        OMX_STATETYPE eState = OMX_StateInvalid;
        OMX_GetState(self->decoder.hComponent, &eState);
        if(eState == OMX_StateExecuting) {
            /* Send output buffers */
            OMX_S32 i;
            for(i=0; (int)i<self->decoder.nBufferHdr[1]; i++) {
                self->decoder.pBufferHdr[1][i]->nFilledLen = 0;
                self->decoder.pBufferHdr[1][i]->nOffset = 0;
                OMX_FillThisBuffer(self->decoder.hComponent, self->decoder.pBufferHdr[1][i]);
            }
        }
    }
    return OMX_ErrorNone;
}

OMX_ERRORTYPE enc_process_event(tdav_codec_h263_hw_vpu_t *self, MSG *pMsg)
{
    //TSK_DEBUG_INFO("enc_process_event IN\n");
    OMX_EVENTTYPE event = pMsg->data.event.eEvent;
    OMX_U32 nPortIndex = pMsg->data.event.nData1;

    if(event == OMX_EventPortSettingsChanged) {
        enc_port_disable(self, nPortIndex);
        enc_port_enable(self, nPortIndex);

        OMX_STATETYPE eState = OMX_StateInvalid;
        OMX_GetState(self->encoder.hComponent, &eState);
        if(eState == OMX_StateExecuting) {
            /* Send output buffers */
            OMX_S32 i;
            for(i=0; (int)i<self->encoder.nBufferHdr[1]; i++) {
                self->encoder.pBufferHdr[1][i]->nFilledLen = 0;
                self->encoder.pBufferHdr[1][i]->nOffset = 0;
                OMX_FillThisBuffer(self->encoder.hComponent, self->encoder.pBufferHdr[1][i]);
            }
        }
    }
    return OMX_ErrorNone;
}

static void free(tdav_codec_h263_hw_vpu_t *h263)
{
    if (!h263) return;

    //Encoder process queue
    if (h263->encoder.pMsgQ)
        FSL_FREE(h263->encoder.pMsgQ);

    //Decoder process queue
    if (h263->decoder.pMsgQ)
        FSL_FREE(h263->decoder.pMsgQ);
}

static void ProcessBufferForCrop(tdav_codec_h263_hw_vpu_t* self, uint8_t* srcPtr, uint8_t* desPtr)
{
    if ( (!self) || (!srcPtr) || (!desPtr)) {
        TSK_DEBUG_ERROR("invalid self ptr or input or output buffer");
        return;
    }

    int cropLeft = self->decoder.cropLeft;
    int cropTop = self->decoder.cropTop;
    int cropRight = self->decoder.cropRight;
    int cropBottom = self->decoder.cropBottom;

    uint8_t *buf, *out_data_ptr, *pCropYuv;
    int cropWidth = 0, cropHeight = 0, i = 0;

    buf = srcPtr;
    out_data_ptr = desPtr;

    int width = self->decoder.context->width;
    int height = self->decoder.context->height;

    pCropYuv = buf;
    cropWidth = cropRight - cropLeft;
    cropHeight = cropBottom - cropTop;

    pCropYuv += width * cropTop;
    pCropYuv += cropLeft;

    for (i = 0; i < cropHeight; i++) {
        if (!out_data_ptr || !pCropYuv) {
            TSK_DEBUG_ERROR("ProcessBufferForCrop:1 buffer invalidated.");
            return;
        }
        memcpy(out_data_ptr, pCropYuv, cropWidth);
        out_data_ptr += cropWidth;
        pCropYuv += width;
    }

    cropWidth /= 2;
    cropHeight /= 2;

    pCropYuv = buf + (width * height);
    pCropYuv += (width / 2) * (cropTop / 2);
    pCropYuv += cropLeft / 2;

    for (i = 0; i < cropHeight; i++) {
        if (!out_data_ptr || !pCropYuv) {
            TSK_DEBUG_ERROR("ProcessBufferForCrop:2 buffer invalidated.");
            return;
        }
        memcpy(out_data_ptr, pCropYuv, cropWidth);
        out_data_ptr += cropWidth;
        pCropYuv += width / 2;
    }

    pCropYuv = buf + (width * height) * 5 / 4;
    pCropYuv += (width / 2) * (cropTop / 2);
    pCropYuv += cropLeft / 2;

    for (i = 0; i < cropHeight; i++) {
        if (!out_data_ptr || !pCropYuv) {
            TSK_DEBUG_ERROR("ProcessBufferForCrop:3 buffer invalidated.");
            return;
        }
        memcpy(out_data_ptr, pCropYuv, cropWidth);
        out_data_ptr += cropWidth;
        pCropYuv += width / 2;
    }
}

/*
 * This function is to convert framebuffer from interleaved Cb/Cr mode
 * to non-interleaved Cb/Cr mode.
 *
 * Note: This function does _NOT_ really store this framebuffer into file.
 */
void saveNV12ImageHelper(tdav_codec_h263_hw_vpu_t* self, uint8_t *pYuv, uint8_t *buf)
{
    TSK_DEBUG_INFO("saveNV12ImageHelper IN");
    int Y, Cb;
    uint8_t *Y1, *Cb1, *Cr1;
    int img_size;
    int y, x;
    uint8_t *tmp;
    int height = self->decoder.context->height;
    int width = self->decoder.context->width;

    if (!pYuv || !buf) {
        TSK_DEBUG_ERROR("pYuv or buf should not be NULL.\n");
            return;
    }

    img_size = width * height;

    Y = (int)buf;
    Cb = Y + img_size;

    Y1 = pYuv;
    Cb1 = Y1 + img_size;
    Cr1 = Cb1 + (img_size >> 2);

    memcpy(Y1, (uint8_t *)Y, img_size);

    for (y = 0; y < (height / 2); y++) {
        tmp = (uint8_t *)(Cb + width * y);
        for (x = 0; x < width; x += 2) {
            *Cb1++ = tmp[x];
            *Cr1++ = tmp[x + 1];
        }
    }
    TSK_DEBUG_INFO("saveNV12ImageHelper OUT");
}

// Not in use. Use if video eColorFormat is 420SP
static void ProcessBuffer(tdav_codec_h263_hw_vpu_t* self, uint8_t* srcPtr, uint8_t* desPtr, int size)
{
    int height = self->decoder.context->height;
    int width = self->decoder.context->width;
    uint8_t *pYuv = NULL, *pYuv0 = NULL, *buf;

    /*
    * There could be these three cases:
    * interleave == 0, cropping == 1
    * interleave == 1, cropping == 0
    * interleave == 1, cropping == 1
    */
    if (self->decoder.chromaInterleave) {
        TSK_DEBUG_INFO("ProcessBuffer: chromaInterleave enabled");
        if (!pYuv) {
            pYuv0 = pYuv = (uint8_t*) tsk_malloc(size);
            if (!pYuv) {
                TSK_DEBUG_ERROR("malloc error\n");
                return;
            }
        }
        saveNV12ImageHelper(self, pYuv, buf);
    }

    if (!pYuv0) {
        ProcessBufferForCrop(self, srcPtr, desPtr);
    } else {/* process the buf beginning at pYuv0 again for Crop. */
        pYuv = pYuv0;
        ProcessBufferForCrop(self, pYuv, desPtr);
    }

    if (pYuv0) {
        TSK_FREE(pYuv0);
        pYuv0 = NULL;
        pYuv = NULL;
    }
}

/* Decoder config changed, re-calculate decode frame size,
 * update Doubango with changed resolution,
 * re-allocate crop buffer,
 * re-allocate input and output decoder ring buffer with changed frame size
*/
static int reset_decoder_config(tdav_codec_h263_hw_vpu_t *h263, int width, int height)
{
    if (!h263) {
        TSK_DEBUG_ERROR("reset_decoder_config: Invalid parameter");
        return -1;
    }

    int in_width = Align(TMEDIA_CODEC_VIDEO(h263)->in.width, FRAME_ALIGN);
    int in_height = Align(TMEDIA_CODEC_VIDEO(h263)->in.height, FRAME_ALIGN);
    int dec_width = h263->decoder.context->width;
    int dec_height = h263->decoder.context->height;
    int max_buffer_size = 0;
    int max_capacity = 0;

    TSK_DEBUG_INFO("reset_decoder_config: old res (%dx%d), new res (%dx%d)",
          in_width, in_height, dec_width, dec_height);

    if ((dec_width != in_width) || (dec_height != in_height)) {
        TSK_DEBUG_WARN("reset_decoder_config: Need to reconfigure decoder settings");
        h263->decoder.decode_frame_size = dec_width * dec_height * 3 / 2;
        TSK_DEBUG_WARN("reset_decoder_config: width, height (%d, %d),dec size %d",
             dec_width, dec_height, h263->decoder.decode_frame_size);

        // Update Doubango's width and height without alignment
        TMEDIA_CODEC_VIDEO(h263)->in.width = dec_width;
        TMEDIA_CODEC_VIDEO(h263)->in.height = dec_height;

        if (!(h263->decoder.crop_buffer = tsk_realloc(h263->decoder.crop_buffer, h263->decoder.decode_frame_size))) {
            TSK_DEBUG_ERROR("Failed to allocate decoder crop buffer");
            return -1;
        }

        // Free decoder input ring buffer
        ring_buffer_free(h263->decoder.in_ring_buf);

        // re-allocate decoder input ring buffer
        max_buffer_size = (dec_width * dec_height *3) >> 3;
        max_capacity = h263->decoder.in_ring_buf_cap;
        h263->decoder.in_ring_buf = ring_buffer_init(max_capacity, max_buffer_size);

        if (h263->decoder.in_ring_buf == NULL) {
            TSK_DEBUG_ERROR("Error allocating input ring buffer for decoder");
            return -1;
        }

        // Free decoder out ring buffer
        ring_buffer_free(h263->decoder.out_ring_buf);

        // re-allocate decoder output ring buffer
        max_buffer_size = (h263->decoder.context->width * h263->decoder.context->height *3) >> 1;
        max_capacity = h263->decoder.out_ring_buf_cap;
        h263->decoder.out_ring_buf = ring_buffer_init(max_capacity, max_buffer_size);
        TSK_DEBUG_INFO("h263->decoder.out_ring_buf %p", h263->decoder.out_ring_buf);

        if (h263->decoder.out_ring_buf == NULL) {
            TSK_DEBUG_ERROR("Error allocating output ring buffer for decoder");
            return -1;
        }
    }
    return 0;
}

/* Close and re-open decoder
*/
int reset_decoder(tdav_codec_h263_hw_vpu_t *h263)
{
    TSK_DEBUG_INFO("reset_decoder");
    if (!h263) {
        TSK_DEBUG_ERROR("reset_decoder: invalid parameter");
        return -1;
    }

    if (!h263->decoder.decoder_reset) {
        // Set flag to indicate decoder reset is started
        h263->decoder.decoder_reset = tsk_true;
    } else {
        TSK_DEBUG_WARN("reset_decoder: decoder reset already started");
        return 0;
    }

    // Free resources
#if 0
    if (h263->decoder.prev_frame) {
        FSL_FREE(h263->decoder.prev_frame);
        h263->decoder.prev_frame = NULL;
    }
#endif
    StateTrans(h263, OMX_StateIdle);
    StateTrans(h263, OMX_StateLoaded);
    unload_component(h263);
    h263->decoder.bClosedDecoder = OMX_TRUE;

    TSK_DEBUG_INFO("reset_decoder: opening OMX decoder");
    return open_fsl_omx_decoder(h263);
}

static void start_reset_dec_timer(tdav_codec_h263_hw_vpu_t *h263)
{
    TSK_DEBUG_INFO("start_reset_dec_timer");
    int ret;

    if (!h263) {
        TSK_DEBUG_ERROR("Invalid parameter");
        return;
    }

    // Create timer for pasuing codec after timeout period
    if (!(h263->h_timer = tsk_timer_manager_create())) {
        TSK_DEBUG_ERROR("Failed to create timer manager for h263");
        //ignore this error
        return;
    }

    if ((ret = tsk_timer_manager_start(h263->h_timer)) != 0) {
        TSK_DEBUG_ERROR("Failed to start the timer");
        //Can ignore this error
        return;
    }

    h263->timer_started = (ret == 0);

    if (h263->timer_started) {
        h263->timer_reset_dec.timeout = h263->reset_timeout_period;
        h263->timer_reset_dec.id = tsk_timer_manager_schedule(h263->h_timer, h263->timer_reset_dec.timeout, _tdav_session_h263_timer_cb, h263);
        TSK_DEBUG_INFO("Dec reset timer sheduled with timeout value %d ms", h263->reset_timeout_period);
    }
}

OMX_ERRORTYPE request_intraVOP(tdav_codec_h263_hw_vpu_t *self)
{
    if (!self) {
        TSK_DEBUG_ERROR("request_intraVOP: invalid parameter");
        return OMX_ErrorUndefined;
    }

    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_CONFIG_INTRAREFRESHVOPTYPE vop;

    OMX_INIT_STRUCT(&vop, OMX_CONFIG_INTRAREFRESHVOPTYPE);
    vop.nPortIndex = (OMX_U32) kPortIndexOutput;
    vop.IntraRefreshVOP = OMX_TRUE;

    ret = OMX_SetConfig(self->encoder.hComponent, OMX_IndexConfigVideoIntraVOPRefresh, &vop);
    if (ret != OMX_ErrorNone) {
        TSK_DEBUG_ERROR("request_intraVOP: Unable to set IntraVOPRefresh for %d", kPortIndexOutput);
    }
    TSK_DEBUG_INFO("request_intraVOP: Set IntraVOPRefresh %d", vop.IntraRefreshVOP);
    return ret;
}

bool set_bitrate_fac_GOP_size(tdav_codec_h263_hw_vpu_t *self)
{
    TSK_DEBUG_INFO("set_bitrate_fac_GOP_size");
    if (!self) {
        TSK_DEBUG_ERROR("set_bitrate_fac_GOP_size: invalid parameter");
        return false;
    }

    int in_height = TMEDIA_CODEC_VIDEO(self)->in.height;
    char propValue[PROPERTY_VALUE_MAX];
    int value = 0;

    if (in_height == HT_720P) {
        self->encoder.bitrate_fact = MED_MOTION;
        self->encoder.gop_size = LOW_GOP_SIZE_IN_SEC;
        if (property_get("bit_rate_fac_hd", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_BR_FACT) {
                self->encoder.bitrate_fact = value;
            }
        }
        if (property_get("gop_size_hd", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_GOP_SIZE_ALLOWED) {
                self->encoder.gop_size = value;
            }
        }
    } else if (in_height == HT_VGA) {
        self->encoder.bitrate_fact = MED_MOTION;
        self->encoder.gop_size = HIGH_GOP_SIZE_IN_SEC;
        if (property_get("bit_rate_fac_vga", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_BR_FACT) {
                self->encoder.bitrate_fact = value;
            }
        }
        if (property_get("gop_size_vga", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_GOP_SIZE_ALLOWED) {
                self->encoder.gop_size = value;
            }
        }
    } else if (in_height == HT_QVGA) {
        self->encoder.bitrate_fact = HIGH_MOTION;
        self->encoder.gop_size = HIGH_GOP_SIZE_IN_SEC;
        if (property_get("bit_rate_fac_qvga", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_BR_FACT) {
                self->encoder.bitrate_fact = value;
            }
        }
        if (property_get("gop_size_qvga", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_GOP_SIZE_ALLOWED) {
                self->encoder.gop_size = value;
            }
        }
    } else {
        self->encoder.bitrate_fact = MED_MOTION;
        self->encoder.gop_size = DEF_GOP_SIZE_IN_SEC;
        if (property_get("bit_rate_fac_default", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_BR_FACT) {
                self->encoder.bitrate_fact = value;
            }
        }
        if (property_get("gop_size_default", propValue, NULL)) {
            value = atoi(propValue);
            if (value >= 0 && value < MAX_GOP_SIZE_ALLOWED) {
                self->encoder.gop_size = value;
            }
        }
    }
    return true;
}
#endif /* HAVE_FFMPEG */

