package org.mortbay.ijetty;

import org.mortbay.ijetty.util.IJettyToast;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import com.maxwell.provisioning.util.common;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import com.maxwell.shared.SharedMaxwellUtils;








/**
 * WebServerDeviceStartUpReceiver
 *
 * Receiver to start IJETTY at boot
 */

public class WebServerDeviceStartUpReceiver extends BroadcastReceiver{
    private final String TAG =   "WebServerDeviceStartUpReceiver";

    @Override
     public void onReceive(Context context, Intent intent)
     {
         if (Common.__DEBUG)Log.i(TAG,"onReceive <<");
         setWebUIVersion(context);
         if (Intent.ACTION_BOOT_COMPLETED.equals(intent.getAction())) {
             int state = common.getPersistentJettyState(context);
                 if (Common.__DEBUG)Log.d(TAG, "jetty state" + state);
                 if(common.WEBUI_STOPPED != state) {
                    common.setJettyState(context, common.WEBUI_STOPPED);
                    Common.startJetty(context, new Intent(context, IJettyService.class));
                 } else{
                    IJettyToast.showServiceToast(context,R.string.jetty_is_currently_off);
                    int mode = common.getUserMode(context);
                    common.setServerMode(context, mode);
                 }
         }else if (ConnectivityManager.CONNECTIVITY_ACTION.equals(intent.getAction())) {
            if(common.WEBUI_STARTED != common.getPersistentJettyState(context))
                return;
            NetworkInfo netInfoWifi =  ((ConnectivityManager)context.getSystemService(context.CONNECTIVITY_SERVICE)).getActiveNetworkInfo();
            checkAndRestartJetty(netInfoWifi, context);

         }
         else{
            Log.e(TAG,"Error: No event matched");
         }
     }



    private void checkAndRestartJetty(NetworkInfo netInfoWifi, Context context){
             if (Common.__DEBUG)Log.i(TAG,"checkAndRestartJetty");
            if (null != netInfoWifi  && netInfoWifi.isConnected()){
                String newip = SharedMaxwellUtils.getNetworkIP(netInfoWifi, context);
                String oldip = Common.getWebServerIP();

                if (Common.__DEBUG)Log.i(TAG,"checkAndRestartJetty new ip = "+newip +",old IP = "+oldip);
                if (null == newip)
                    return;
                if (null == oldip){
                   Common.setWebServerIP(newip);
                   return;//First time network connection after boot
                }
                if (!newip.matches(oldip)){
                    Common.setWebServerIP(newip);
                    Common.restartJetty(context,new Intent(context, IJettyService.class));
                }else{
                    int mode = common.getUserMode(context);
                    common.setServerMode(context, mode);
                }

            }
    }
    private void setWebUIVersion(Context context){
        try{
           PackageManager manager = context.getPackageManager();
           PackageInfo info = manager.getPackageInfo(context.getPackageName(), 0);
           common.setWebUIVersion(info.versionName);
         }catch(Exception e){
           e.printStackTrace();
         }
    }
}
